@@ -91,6 +91,11 @@
 #define GARMIN_PKTID_L001_RECORDS        27
 #define GARMIN_PKTID_L001_WPT_DATA       35
 
+#define	CMND_ABORT			 0
+#define	CMND_START_PVT_DATA		 49
+#define	CMND_STOP_PVT_DATA		 50
+#define	CMND_START_RM_DATA		 110
+
 #define MAX_BUFFER_SIZE 4096
 
 #define GARMIN_CHANNELS	12
@@ -202,22 +207,25 @@ static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt );
 static void SendPacket (struct gps_device_t *session, Packet_t *aPacket );
 static int GetPacket (struct gps_device_t *session );
 
+/*@ -branchstate @*/
 // For debugging, decodes and prints some known packets.
 static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 {
     gps_mask_t mask = 0;
     int maj_ver;
     int min_ver;
-    unsigned int mode;
-    unsigned short prod_id;
-    unsigned short ver;
-    unsigned int veri;
+    unsigned int mode = 0;
+    unsigned short prod_id = 0;
+    unsigned short ver = 0;
+    unsigned int veri = 0;
     time_t time_l = 0;
     unsigned int serial;
     cpo_sat_data *sats = NULL;
     cpo_pvt_data *pvt = NULL;
     int i = 0, j = 0;
     double track;
+    char *msg = NULL;
+    char buf[40] = "";
 
     gpsd_report(3, "PrintPacket()\n");
     if ( 4096 < pkt->mDataSize) {
@@ -234,7 +242,9 @@ static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 	    gpsd_report(3, "Transport, Start Session req\n");
 	    break;
 	case GARMIN_PKTID_TRANSPORT_START_SESSION_RESP:
-	    gpsd_report(3, "Transport, Start Session resp\n");
+	    mode = get_int(&pkt->mData.uchars[0]);
+	    gpsd_report(3, "Transport, Start Session resp, unit: 0x%x\n"
+		, mode);
 	    break;
 	default:
 	    gpsd_report(3, "Transport, Packet: Type %d %d %d, ID: %d, Sz: %d\n"
@@ -248,6 +258,28 @@ static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 	break;
     case GARMIN_LAYERID_APPL:
 	switch( pkt->mPacketId ) {
+	case GARMIN_PKTID_L001_COMMAND_DATA:
+	    prod_id = get_short(&pkt->mData.uchars[0]);
+            switch ( prod_id ) {
+	    case CMND_ABORT:
+		msg = "Abort current xfer";
+	  	break;
+	    case CMND_START_PVT_DATA:
+		msg = "Start Xmit PVT data";
+	  	break;
+	    case CMND_STOP_PVT_DATA:
+		msg = "Stop Xmit PVT data";
+	  	break;
+	    case CMND_START_RM_DATA:
+		msg = "Start RMD data";
+	  	break;
+	    default:
+		(void)snprintf(buf, sizeof(buf), "Unknown: %u", prod_id);
+		msg = buf;
+	        break;
+            }
+	    gpsd_report(3, "Appl, Command Data: %s\n", msg);
+	    break;
 	case GARMIN_PKTID_PRODUCT_RQST:
 	    gpsd_report(3, "Appl, Product Data req\n");
 	    break;
@@ -290,10 +322,10 @@ static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt)
             // gpsd sign is opposite of garmin sign
 	    session->gpsdata.separation = -pvt->msl_hght;
 
-	    // estimated position error in meters
-	    session->gpsdata.epe = pvt->epe;
-	    session->gpsdata.newdata.eph = pvt->eph;
-	    session->gpsdata.newdata.epv = pvt->epv;
+	    // Estimated position error in meters.
+	    session->gpsdata.epe = pvt->epe * (GPSD_CONFIDENCE/2);
+	    session->gpsdata.newdata.eph = pvt->eph * (GPSD_CONFIDENCE/2);
+	    session->gpsdata.newdata.epv = pvt->epv * (GPSD_CONFIDENCE/2);
 
 	    // convert lat/lon to knots
 	    session->gpsdata.newdata.speed
@@ -385,9 +417,7 @@ static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 			    , sats->azmth
 			    , sats->status);
 
-		// busted??
-// FIXME!! need to elimate sats over 32
-		if ( (32 < (int)sats->svid) || (0 == (int)sats->svid) ) {
+		if ( 255 == (int)sats->svid ) {
 		    // Garmin uses 255 for empty
 		    // gpsd uses 0 for empty
 		    continue;
@@ -408,12 +438,12 @@ static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 		session->gpsdata.satellites++;
 		j++;
 	    }
-	    mask |= SATELLITE_SET;
+	    mask |= SATELLITE_SET | USED_SET;
 	    break;
 	case GARMIN_PKTID_PROTOCOL_ARRAY:
             // this packet is never requested, it just comes, in some case
             // after a GARMIN_PKTID_PRODUCT_RQST 
-	    gpsd_report(3, "Appl, Product Capability: %d\n", pkt->mDataSize);
+	    gpsd_report(3, "Appl, Product Capability, sz: %d\n", pkt->mDataSize);
             for ( i = 0; i < (int)pkt->mDataSize ; i += 3 ) {
 		    gpsd_report(3, "  %c%03d\n", pkt->mData.chars[i]
 			, get_short( &(pkt->mData.uchars[i+1])) );
@@ -429,6 +459,10 @@ static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt)
     case 75:
 	// private
 	switch( pkt->mPacketId ) {
+	case PRIV_PKTID_SET_MODE:
+	    prod_id = (unsigned short)get_int(&pkt->mData.uchars[0]);
+	    gpsd_report(3, "Private, Set Mode: %d\n", prod_id);
+	    break;
 	case PRIV_PKTID_INFO_REQ:
 	    gpsd_report(3, "Private, ID: Info Req\n");
 	    break;
@@ -461,6 +495,7 @@ static gps_mask_t PrintPacket(struct gps_device_t *session, Packet_t *pkt)
 
     return mask;
 }
+/*@ +branchstate @*/
 
 //-----------------------------------------------------------------------------
 // send a packet in GarminUSB format
@@ -514,9 +549,10 @@ static int GetPacket (struct gps_device_t *session )
     int cnt = 0;
     // int x = 0; // for debug dump
 
-    memset( session->garmin.Buffer, 0, sizeof(Packet_t));
+    memset( session->driver.garmin.Buffer, 0, sizeof(Packet_t));
     memset( &delay, 0, sizeof(delay));
-    session->garmin.BufferLen = 0;
+    session->driver.garmin.BufferLen = 0;
+    session->outbuflen = 0;
 
     gpsd_report(4, "GetPacket()\n");
 
@@ -527,10 +563,10 @@ static int GetPacket (struct gps_device_t *session )
 	// not optimal, but given the speed and packet nature of
 	// the USB not too bad for a start
 	ssize_t theBytesReturned = 0;
-	unsigned char *buf = session->garmin.Buffer;
+	unsigned char *buf = session->driver.garmin.Buffer;
 
 	theBytesReturned = read(session->gpsdata.gps_fd
-		, buf + session->garmin.BufferLen
+		, buf + session->driver.garmin.BufferLen
 		, ASYNC_DATA_SIZE);
         if ( 0 >  theBytesReturned ) {
 	    // read error...
@@ -541,15 +577,15 @@ static int GetPacket (struct gps_device_t *session )
 	}
 	gpsd_report(5, "got %d bytes\n", theBytesReturned);
 
-	session->garmin.BufferLen += theBytesReturned;
+	session->driver.garmin.BufferLen += theBytesReturned;
 	if ( 64 > theBytesReturned ) {
 	    // zero length, or short, read is a flag for got the whole packet
             break;
 	}
 		
-	if ( 256 <=  session->garmin.BufferLen ) {
+	if ( 256 <=  session->driver.garmin.BufferLen ) {
 	    // really bad read error...
-	    session->garmin.BufferLen = 0;
+	    session->driver.garmin.BufferLen = 0;
 	    gpsd_report(3, "GetPacket() packet too long!\n");
 	    break;
 	}
@@ -562,15 +598,16 @@ static int GetPacket (struct gps_device_t *session )
 	/*@ end @*/
     }
     // dump the individual bytes, debug only
-    // for ( x = 0; x < session->garmin.BufferLen; x++ ) {
-        // gpsd_report(6, "p[%d] = %x\n", x, session->garmin.Buffer[x]);
+    // for ( x = 0; x < session->driver.garmin.BufferLen; x++ ) {
+        // gpsd_report(6, "p[%d] = %x\n", x, session->driver.garmin.Buffer[x]);
     // }
     if ( 10 <= cnt ) {
 	    gpsd_report(3, "GetPacket() packet too long or too slow!\n");
 	    return -1;
     }
 
-    gpsd_report(5, "GotPacket() sz=%d \n", session->garmin.BufferLen);
+    gpsd_report(5, "GotPacket() sz=%d \n", session->driver.garmin.BufferLen);
+    session->outbuflen = session->driver.garmin.BufferLen;
     return 0;
 }
 
@@ -614,16 +651,16 @@ static bool garmin_probe(struct gps_device_t *session)
     }
 
     /* reset the buffer and buffer length */
-    memset( session->garmin.Buffer, 0, sizeof(session->garmin.Buffer) );
-    session->garmin.BufferLen = 0;
+    memset( session->driver.garmin.Buffer, 0, sizeof(session->driver.garmin.Buffer) );
+    session->driver.garmin.BufferLen = 0;
 
-    if (sizeof(session->garmin.Buffer) < sizeof(Packet_t)) {
+    if (sizeof(session->driver.garmin.Buffer) < sizeof(Packet_t)) {
 	gpsd_report(0, "garmin_probe: Compile error, garmin.Buffer too small.\n",
              strerror(errno));
 	return false;
     }
 
-    buffer = session->garmin.Buffer;
+    buffer = session->driver.garmin.Buffer;
     thePacket = (Packet_t*)buffer;
 
     // set Mode 0
@@ -644,13 +681,13 @@ static bool garmin_probe(struct gps_device_t *session)
 
     SendPacket(session,  thePacket);
 
-    // get and print the driver Version info
+    /* get and print the driver Version info */
 
     FD_ZERO(&fds); 
     FD_SET(session->gpsdata.gps_fd, &fds);
 
-    // Wait, nicely, until the device returns the Version info
-    // Toss any other packets, up to 4
+    /* Wait, nicely, until the device returns the Version info
+     * Toss any other packets, up to 4 */
     ok = 0;
     memset( &tv,0,sizeof(tv));
     for( i = 0 ; i < 4 ; i++ ) {
@@ -664,7 +701,7 @@ static bool garmin_probe(struct gps_device_t *session)
 	    gpsd_report(0, "select: %s\n", strerror(errno));
 	    return false;
 	} else if ( sel_ret == 0 ) {
-	    gpsd_report(3, "garmin_probe() timeout\n");
+	    gpsd_report(3, "garmin_probe() timeout, INFO_REQ\n");
 	    // restore old terminal settings
             (void)tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH
 		, &session->ttyset_old);
@@ -688,7 +725,10 @@ static bool garmin_probe(struct gps_device_t *session)
 		, &session->ttyset_old);
 	return false;
     }
-    // Tell the device that we are starting a session.
+    /* depending on the GARMIN version, the device may spontaneously
+       return the Product Capability here */
+
+    /* Tell the device that we are starting a session. */
     gpsd_report(3, "Send Garmin Start Session\n");
 
     set_int(buffer, GARMIN_LAYERID_TRANSPORT);
@@ -697,8 +737,8 @@ static bool garmin_probe(struct gps_device_t *session)
 
     SendPacket(session,  thePacket);
 
-    // Wait until the device is ready to the start the session
-    // Toss any other packets, up to 4
+    /* Wait until the device is ready to the start the session
+     * Toss any other packets, up to 4 */
     ok = 0;
     for( i = 0 ; i < 4 ; i++ ) {
         memcpy((char *)&rfds, (char *)&fds, sizeof(rfds));
@@ -711,13 +751,14 @@ static bool garmin_probe(struct gps_device_t *session)
 	    gpsd_report(0, "select: %s\n", strerror(errno));
 	    return(0);
 	} else if ( sel_ret == 0 ) {
-	    gpsd_report(3, "garmin_probe() timeout\n");
+	    gpsd_report(3, "garmin_probe() timeout, START_SESSION\n");
 	    // restore old terminal settings
             (void)tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH
 		, &session->ttyset_old);
 	    return(0);
         }
 	if ( 0 == GetPacket( session ) ) {
+	    gpsd_report(3, "Got packet waiting for START_SESSION\n");
 	    (void)PrintPacket(session, thePacket);
 
 	    if( (GARMIN_LAYERID_TRANSPORT == thePacket->mPacketType)
@@ -728,6 +769,7 @@ static bool garmin_probe(struct gps_device_t *session)
 	    }
 	}
     }
+
     if ( 0 == ok ) {
 	gpsd_report(2, "Garmin driver never answered to START_SESSION.\n");
 	// restore old terminal settings
@@ -759,7 +801,7 @@ static bool garmin_probe(struct gps_device_t *session)
 	    gpsd_report(0, "select: %s\n", strerror(errno));
 	    return false;
 	} else if ( sel_ret == 0 ) {
-	    gpsd_report(3, "garmin_probe() timeout\n");
+	    gpsd_report(3, "garmin_probe() timeout, PRODUCT_DATA\n");
 	    // restore old terminal settings
             (void)tcsetattr(session->gpsdata.gps_fd, TCIOFLUSH
 		, &session->ttyset_old);
@@ -798,12 +840,14 @@ static bool garmin_probe(struct gps_device_t *session)
  */
 static void garmin_init(struct gps_device_t *session)
 {
-	unsigned char *buffer = session->garmin.Buffer;
+	unsigned char *buffer = session->driver.garmin.Buffer;
 	Packet_t *thePacket = (Packet_t*)buffer;
 	bool ret;
 
 	gpsd_report(5, "to garmin_probe()\n");
 	ret = garmin_probe( session );
+        /* FIXME - what if return code was bad */
+        /* FIXME - return code is always bad */
 	gpsd_report(3, "from garmin_probe() = %d\n", (int)ret);
 
 	// turn on PVT data 49
@@ -812,7 +856,7 @@ static void garmin_init(struct gps_device_t *session)
 	set_int(buffer, GARMIN_LAYERID_APPL);
 	set_int(buffer+4, GARMIN_PKTID_L001_COMMAND_DATA);
 	set_int(buffer+8, 2); // data length 2
-	set_int(buffer+12, 49); //  49, CMND_START_PVT_DATA
+	set_int(buffer+12, CMND_START_PVT_DATA);
 
 	SendPacket(session,  thePacket);
 
@@ -820,11 +864,19 @@ static void garmin_init(struct gps_device_t *session)
 	//set_int(buffer, GARMIN_LAYERID_APPL);
 	//set_int(buffer+4, GARMIN_PKTID_L001_COMMAND_DATA);
 	//set_int(buffer+8, 2); // data length 2
-	//set_int(buffer+12, 110); // 110, CMND_START_ Rcv Measurement Data
+	//set_int(buffer+12, CMND_START_RM_DATA);
 
 	//SendPacket(session,  thePacket);
 }
 
+static void garmin_close(struct gps_device_t *session) 
+{
+    /* FIXME -- do we need to put the garmin to sleep?  or is closing the port
+       sufficient? */
+    gpsd_report(3, "garmin_close()\n");
+    return;
+}
+
 static ssize_t garmin_get_packet(struct gps_device_t *session) 
 {
     return (ssize_t)( 0 == GetPacket( session ) ? 1 : 0);
@@ -832,23 +884,27 @@ static ssize_t garmin_get_packet(struct gps_device_t *session)
 
 static gps_mask_t garmin_parse_input(struct gps_device_t *session)
 {
-    return PrintPacket(session, (Packet_t*)session->garmin.Buffer);
+    gpsd_report(5, "garmin_parse_input()\n");
+    return PrintPacket(session, (Packet_t*)session->driver.garmin.Buffer);
 }
 
 /* this is everything we export */
 struct gps_type_t garmin_binary =
 {
-    "Garmin binary",	/* full name of type */
-    NULL,		/* only switched to by some other driver */
-    garmin_probe,	/* how to detect this at startup time */
-    garmin_init,	/* initialize the device */
-    garmin_get_packet,	/* how to grab a packet */
-    garmin_parse_input,	/* read and parse message packets */
-    NULL,		/* send DGPS correction */
-    NULL,		/* no speed switcher */
-    NULL,		/* no mode switcher */
-    NULL,		/* caller needs to supply a close hook */
-    1,			/* updates every second */
+    .typename       = "Garmin binary",	/* full name of type */
+    .trigger        = NULL,		/* no trigger, it has a probe */
+    .channels       = GARMIN_CHANNELS,	/* consumer-grade GPS */
+    .probe          = garmin_probe,	/* how to detect at startup time */
+    .initializer    = garmin_init,	/* initialize the device */
+    .get_packet     = garmin_get_packet,/* how to grab a packet */
+    .parse_packet   = garmin_parse_input,	/* parse message packets */
+    .rtcm_writer    = NULL,		/* don't send DGPS corrections */
+    .speed_switcher = NULL,		/* no speed switcher */
+    .mode_switcher  = NULL,		/* no mode switcher */
+    .rate_switcher  = NULL,		/* no sample-rate switcher */
+    .cycle_chars    = -1,		/* not relevant, no rate switch */
+    .wrapup         = garmin_close,	/* close hook */
+    .cycle          = 1,		/* updates every second */
 };
 
 #endif /* GARMIN_ENABLE */
