@@ -35,9 +35,6 @@
 #if defined(HAVE_SYS_TIME_H)
 #include <sys/time.h>
 #endif
-#if defined(PPS_ENABLE)
-#include <pthread.h>
-#endif
 
 #if DBUS_ENABLE
 #include <gpsd_dbus.h>
@@ -78,14 +75,21 @@ static int debuglevel;
 static bool in_background = false;
 static jmp_buf restartbuf;
 /*@ -initallelements -nullassign -nullderef @*/
-static struct gps_context_t context = {0, 
-				       false, 0, -1, 0, {'\0'}, 0,
-				       LEAP_SECONDS, CENTURY_BASE, 
+static struct gps_context_t context = {
+    .valid        = 0, 
+    .sentdgps     = false, 
+    .fixcnt       = 0, 
+    .dsock        = -1, 
+    .rtcmbytes    = 0, 
+    .rtcmbuf      = {'\0'}, 
+    .rtcmtime     = 0,
+    .leap_seconds = LEAP_SECONDS, 
+    .century      = CENTURY_BASE, 
 #ifdef NTPSHM_ENABLE
-				       {0},
-				       {0},
+    .shmTime      = {0},
+    .shmTimeInuse = {0},
 # ifdef PPS_ENABLE
-					0,
+    .shmTimePPS   = false,
 # endif /* PPS_ENABLE */
 #endif /* NTPSHM_ENABLE */
 };
@@ -134,7 +138,7 @@ void gpsd_report(int errlevel, const char *fmt, ... )
 /* assemble command in printf(3) style, use stderr or syslog */
 {
     if (errlevel <= debuglevel) {
-	char buf[BUFSIZ];
+	char buf[BUFSIZ], buf2[BUFSIZ], *sp;
 	va_list ap;
 
 #if defined(PPS_ENABLE)
@@ -145,10 +149,17 @@ void gpsd_report(int errlevel, const char *fmt, ... )
 	(void)vsnprintf(buf + strlen(buf), sizeof(buf)-strlen(buf), fmt, ap);
 	va_end(ap);
 
+	buf2[0] = '\0';
+	for (sp = buf; *sp != '\0'; sp++)
+	    if (isprint(*sp) || (isspace(*sp) && (sp[1]=='\0' || sp[2]=='\0')))
+		(void)snprintf(buf2+strlen(buf2), 2, "%c", *sp);
+	    else
+		(void)snprintf(buf2+strlen(buf2), 6, "\\x%02x", (unsigned)*sp);
+
 	if (in_background)
-	    syslog((errlevel == 0) ? LOG_ERR : LOG_NOTICE, "%s", buf);
+	    syslog((errlevel == 0) ? LOG_ERR : LOG_NOTICE, "%s", buf2);
 	else
-	    (void)fputs(buf, stderr);
+	    (void)fputs(buf2, stderr);
 #if defined(PPS_ENABLE)
 	(void)pthread_mutex_unlock(&report_mutex);
 #endif /* PPS_ENABLE */
@@ -275,6 +286,9 @@ static struct subscriber_t {
     bool tied;				/* client set device with F */
     bool watcher;			/* is client in watcher mode? */
     int raw;				/* is client in raw mode? */
+#ifdef RTCM104_SERVICE
+    bool rtcm;				/* is RTCM what he actually wants? */
+#endif /* RTCM104_SERVICE */
     /*@relnull@*/struct gps_device_t *device;	/* device subscriber listens to */
 } subscribers[FD_SETSIZE];		/* indexed by client file descriptor */
 
@@ -369,8 +383,10 @@ static /*@null@*/ struct gps_device_t *open_device(char *device_name,
     struct gps_device_t *chp;
 
     for (chp = channels; chp < channels + MAXDEVICES; chp++)
-	if (!allocated_channel(chp))
+	if (!allocated_channel(chp)){
+            chp->saved_baud = -1;
 	    goto found;
+        }
     return NULL;
 found:
     gpsd_init(chp, &context, device_name);
@@ -397,7 +413,11 @@ static bool assign_channel(struct subscriber_t *user)
 	/* ...connect him to the most recently active device */
 	for(channel = channels; channel<channels+MAXDEVICES; channel++)
 	    if (allocated_channel(channel)) {
-		if (user->device == NULL || channel->gpsdata.sentence_time >= most_recent) {
+		if ((user->device == NULL || channel->gpsdata.sentence_time >= most_recent)
+#ifdef RTCM104_SERVICE
+		    && (user->rtcm == (channel->packet_type == RTCM_PACKET))
+#endif /* RTCM104_SERVICE */
+		    ) {
 		    user->device = channel;
 		    most_recent = channel->gpsdata.sentence_time;
 		}
@@ -431,6 +451,14 @@ static bool assign_channel(struct subscriber_t *user)
 }
 /*@ +branchstate +usedef +globstate @*/
 
+#ifdef RTCM104_SERVICE
+static int handle_dgpsip_request(int cfd UNUSED, char *buf UNUSED, int buflen UNUSED)
+/* interpret a client request; cfd is the socket back to the client */
+{
+    return 0;	/* not actually interpreting these yet */
+}
+#endif /* RTCM104_SERVICE */
+
 static /*@ observer @*/ char *snarfline(char *p, /*@out@*/char **out)
 /* copy the rest of the command line, before CR-LF */
 {
@@ -447,7 +475,22 @@ static /*@ observer @*/ char *snarfline(char *p, /*@out@*/char **out)
     /*@ +temptrans +mayaliasunique @*/
 }
 
-static int handle_request(int cfd, char *buf, int buflen)
+static bool privileged_user(struct subscriber_t *who)
+/* is this user privileged to change the GPS's behavior? */
+{
+    struct subscriber_t *sub;
+    int subscribercount = 0;
+
+    /* grant user privilege if he's the only one on the channel */
+    for (sub = subscribers; 
+	 	sub < subscribers + sizeof(subscribers)/sizeof(subscribers[0]);
+	 	sub++)
+	if (sub->device == who->device)
+	    subscribercount++;
+    return (subscribercount == 1);
+}
+
+static int handle_gpsd_request(int cfd, char *buf, int buflen)
 /* interpret a client request; cfd is the socket back to the client */
 {
     char reply[BUFSIZ], phrase[BUFSIZ], *p, *stash;
@@ -470,7 +513,7 @@ static int handle_request(int cfd, char *buf, int buflen)
 		(void)strcpy(phrase, ",A=?");
 	    break;
 	case 'B':		/* change baud rate (SiRF/Zodiac only) */
-	    if (assign_channel(whoami) && whoami->device->device_type!=NULL && *p=='=') {
+	    if (assign_channel(whoami) && whoami->device->device_type!=NULL && *p=='=' && privileged_user(whoami)) {
 		i = atoi(++p);
 		while (isdigit(*p)) p++;
 		if (whoami->device->device_type->speed_switcher)
@@ -497,21 +540,40 @@ static int handle_request(int cfd, char *buf, int buflen)
 				whoami->device->gpsdata.stopbits);
 		    }
 	    }
-	    if (whoami->device)
+	    if (whoami->device) {
+		if ( whoami->device->gpsdata.parity == 0 ) {
+			/* zero parity breaks the next snprintf */
+			whoami->device->gpsdata.parity = (unsigned)'N';
+		}
 		(void)snprintf(phrase, sizeof(phrase), ",B=%d %d %c %u", 
 		    (int)gpsd_get_speed(&whoami->device->ttyset),
 			9 - whoami->device->gpsdata.stopbits, 
 			(int)whoami->device->gpsdata.parity,
 			whoami->device->gpsdata.stopbits);
-	    else
+	    } else {
 		(void)strcpy(phrase, ",B=?");
+	    }
 	    break;
 	case 'C':
-	    if (assign_channel(whoami) && whoami->device->device_type!=NULL)
-		(void)snprintf(phrase, sizeof(phrase), ",C=%d", 
-			whoami->device->device_type->cycle);
-	    else
+	    if (!assign_channel(whoami) || whoami->device->device_type==NULL)
 		(void)strcpy(phrase, ",C=?");
+	    else {
+		struct gps_type_t *dev = whoami->device->device_type;
+		double mincycle = (dev->cycle_chars * 10.0) / whoami->device->gpsdata.baudrate;
+		if (*p == '=' && privileged_user(whoami)) {
+		    double cycle = strtod(++p, &p);
+		    if (cycle >= mincycle)
+			if (dev->rate_switcher != NULL)
+			    if (dev->rate_switcher(whoami->device, cycle))
+				dev->cycle = cycle;
+		}
+		if (dev->rate_switcher == NULL)
+		    (void)snprintf(phrase, sizeof(phrase), 
+				   ",C=%.2f", dev->cycle);
+		else
+		    (void)snprintf(phrase, sizeof(phrase), 
+				   ",C=%.2f %.2f", dev->cycle, mincycle);
+	    }
 	    break;
 	case 'D':
 	    (void)strcpy(phrase, ",D=");
@@ -519,7 +581,7 @@ static int handle_request(int cfd, char *buf, int buflen)
 		(void)unix_to_iso8601(whoami->device->gpsdata.fix.time, 
 				phrase+3, (int)(sizeof(phrase)-3));
 	    else
-		(void)strcpy(phrase, "?");
+		(void)strcat(phrase, "?");
 	    break;
 	case 'E':
 	    (void)strcpy(phrase, ",E=?");
@@ -576,11 +638,11 @@ static int handle_request(int cfd, char *buf, int buflen)
 		(void)snprintf(phrase, sizeof(phrase), ",M=%d", whoami->device->gpsdata.fix.mode);
 	    break;
 	case 'N':
-	    if (!assign_channel(whoami))
+	    if (!assign_channel(whoami) || whoami->device->device_type == NULL)
 		(void)strcpy(phrase, ",N=?");
 	    else if (!whoami->device->device_type->mode_switcher)
 		(void)strcpy(phrase, ",N=0");
-	    else {
+	    else if (privileged_user(whoami)) {
 		if (*p == '=') ++p;
 		if (*p == '1' || *p == '+') {
 		    whoami->device->device_type->mode_switcher(whoami->device, 1);
@@ -949,7 +1011,11 @@ int main(int argc, char *argv[])
     static int st, csock = -1;
     static gps_mask_t changed;
     static char *dgpsserver = NULL;
-    static char *service = NULL; 
+    static char *gpsd_service = NULL; 
+#ifdef RTCM104_SERVICE
+    static char *rtcm_service = NULL; 
+    static int nsock, rsock = -1;
+#endif /* RTCM104_SERVICE */
     static char *control_socket = NULL;
     struct gps_device_t *device, *channel;
     struct sockaddr_in fsin;
@@ -958,9 +1024,16 @@ int main(int argc, char *argv[])
     bool go_background = true;
     struct timeval tv;
     // extern char *optarg;
+#ifdef RTCM104_ENABLE
+    struct gps_device_t *gps;
+#endif /* RTCM104_ENABLE */
 
     debuglevel = 0;
-    while ((option = getopt(argc, argv, "F:D:S:d:fhNnpP:v")) != -1) {
+    while ((option = getopt(argc, argv, "F:D:S:d:fhNnpP:v"
+#ifdef RTCM104_SERVICE
+			    "R:"
+#endif /* RTCM104_SERVICE */
+		)) != -1) {
 	switch (option) {
 	case 'D':
 	    debuglevel = (int) strtol(optarg, 0, 0);
@@ -971,8 +1044,13 @@ int main(int argc, char *argv[])
 	case 'N':
 	    go_background = false;
 	    break;
+#ifdef RTCM104_SERVICE
+	case 'R':
+	    rtcm_service = optarg;
+	    break;
+#endif /* RTCM104_SERVICE */
 	case 'S':
-	    service = optarg;
+	    gpsd_service = optarg;
 	    break;
 	case 'd':
 	    dgpsserver = optarg;
@@ -1034,14 +1112,25 @@ int main(int argc, char *argv[])
     openlog("gpsd", LOG_PID, LOG_USER);
     gpsd_report(1, "launching (Version %s)\n", VERSION);
     /*@ -observertrans @*/
-    if (!service)
-	service = getservbyname("gpsd", "tcp") ? "gpsd" : DEFAULT_GPSD_PORT;
+    if (!gpsd_service)
+	gpsd_service = getservbyname("gpsd", "tcp") ? "gpsd" : DEFAULT_GPSD_PORT;
     /*@ +observertrans @*/
-    if ((msock = passivesock(service, "tcp", QLEN)) < 0) {
+    if ((msock = passivesock(gpsd_service, "tcp", QLEN)) < 0) {
 	gpsd_report(0,"command socket create failed, netlib error %d\n",msock);
 	exit(2);
     }
-    gpsd_report(1, "listening on port %s\n", service);
+    gpsd_report(1, "listening on port %s\n", gpsd_service);
+#ifdef RTCM104_SERVICE
+    /*@ -observertrans @*/
+    if (!rtcm_service)
+	rtcm_service = getservbyname("rtcm", "tcp") ? "rtcm" : DEFAULT_RTCM_PORT;
+    /*@ +observertrans @*/
+    if ((nsock = passivesock(rtcm_service, "tcp", QLEN)) < 0) {
+	gpsd_report(0,"rtcm socket create failed, netlib error %d\n",nsock);
+	exit(2);
+    }
+    gpsd_report(1, "listening on port %s\n", rtcm_service);
+#endif /* RTCM104_SERVICE */
 
     if (dgpsserver) {
         int dsock = dgpsip_open(&context, dgpsserver);
@@ -1118,6 +1207,9 @@ int main(int argc, char *argv[])
     (void)signal(SIGPIPE, SIG_IGN);
 
     FD_SET(msock, &all_fds);
+#ifdef RTCM104_SERVICE
+    FD_SET(nsock, &all_fds);
+#endif /* RTCM104_SERVICE */
     FD_ZERO(&control_fds);
 
     /* optimization hack to defer having to read subframe data */
@@ -1134,6 +1226,7 @@ int main(int argc, char *argv[])
     for (;;) {
         (void)memcpy((char *)&rfds, (char *)&all_fds, sizeof(rfds));
 
+	gpsd_report(7, "select waits\n");
 	/* 
 	 * Poll for user commands or GPS data.  The timeout doesn't
 	 * actually matter here since select returns whenever one of
@@ -1178,10 +1271,36 @@ int main(int argc, char *argv[])
 		FD_SET(ssock, &all_fds);
 		subscribers[ssock].active = timestamp();
 		subscribers[ssock].tied = false;
+#ifdef RTCM104_SERVICE
+		subscribers[ssock].rtcm = false;
+#endif /* RTCM104_SERVICE */
 	    }
 	    FD_CLR(msock, &rfds);
 	}
 
+#ifdef RTCM104_SERVICE
+	/* also to RTCM client connections */
+	if (FD_ISSET(nsock, &rfds)) {
+	    socklen_t alen = (socklen_t)sizeof(fsin);
+	    /*@i1@*/int ssock = accept(nsock, (struct sockaddr *)&fsin, &alen);
+
+	    if (rsock < 0)
+		gpsd_report(0, "accept: %s\n", strerror(errno));
+	    else {
+		int opts = fcntl(rsock, F_GETFL);
+
+		if (opts >= 0)
+		    (void)fcntl(rsock, F_SETFL, opts | O_NONBLOCK);
+		gpsd_report(3, "client connect on %d\n", rsock);
+		FD_SET(ssock, &all_fds);
+		subscribers[rsock].active = true;
+		subscribers[rsock].tied = false;
+		subscribers[rsock].rtcm = true;
+	    }
+	    FD_CLR(nsock, &rfds);
+	}
+#endif /* RTCM104_SERVICE */
+
 	/* also be open to new control-socket connections */
 	if (csock > -1 && FD_ISSET(csock, &rfds)) {
 	    socklen_t alen = (socklen_t)sizeof(fsin);
@@ -1229,7 +1348,7 @@ int main(int argc, char *argv[])
 		}
 	    }
 
-	    /* pass the current DGPSIP coorection to the GPS if new */
+	    /* pass the current DGPSIP correction to the GPS if new */
 	    dgpsip_relay(channel);
 
 	    /* get data from the device */
@@ -1241,12 +1360,24 @@ int main(int argc, char *argv[])
 		    gpsd_report(3, "packet sniffer failed to sync up\n");
 		    FD_CLR(channel->gpsdata.gps_fd, &all_fds);
 		    gpsd_deactivate(channel);
-		} if ((changed & ONLINE_SET) == 0) {
-		    gpsd_report(3, "GPS is offline\n");
+		} 
+		if ((changed & ONLINE_SET) == 0) {
+/*
+		    gpsd_report(3, "GPS is offline (%lf sec since data)\n", 
+				timestamp() - channel->gpsdata.online);
+*/
 		    FD_CLR(channel->gpsdata.gps_fd, &all_fds);
 		    gpsd_deactivate(channel);
 		    notify_watchers(channel, "GPSD,X=0\r\n");
 		}
+#ifdef RTCM104_ENABLE
+		/* copy each RTCM-104 correction to all GPSes */
+		if ((changed & RTCM_SET) != 0) {
+		    for (gps = channels; gps < channels + MAXDEVICES; gps++)
+			if (gps->device_type != NULL && gps->device_type->rtcm_writer != NULL)
+			    (void)gps->device_type->rtcm_writer(gps, (char *)channel->outbuffer, channel->outbuflen);
+		}
+#endif /* RTCM104_ENABLE */
 	    }
 
 	    for (cfd = 0; cfd < FD_SETSIZE; cfd++) {
@@ -1263,13 +1394,13 @@ int main(int argc, char *argv[])
 			    (void)strcat(cmds, "$");
 		    }
 		    if (cmds[0] != '\0')
-			(void)handle_request(cfd, cmds, (int)strlen(cmds));
+			(void)handle_gpsd_request(cfd, cmds, (int)strlen(cmds));
 		}
 	    }
 #if DBUS_ENABLE
 	    if (changed &~ ONLINE_SET) {
 		    if (changed & (LATLON_SET | MODE_SET)) 
-			    send_dbus_fix (*channel);
+			    send_dbus_fix (channel);
 	    }
 #endif
 	}
@@ -1301,17 +1432,23 @@ int main(int argc, char *argv[])
 		if ((buflen = (int)read(cfd, buf, sizeof(buf) - 1)) <= 0) {
 		    detach_client(cfd);
 		} else {
-		    double now = timestamp();
-
 		    buf[buflen] = '\0';
 		    gpsd_report(1, "<= client: %s", buf);
 
-		    if (subscribers[cfd].device)
-			subscribers[cfd].device->poll_times[cfd] = now;
-		    if (handle_request(cfd, buf, buflen) < 0)
-			detach_client(cfd);
-		    else
-			subscribers[cfd].active = now;
+#ifdef RTCM104_SERVICE
+		    if (subscribers[cfd].rtcm) {
+			if (handle_dgpsip_request(cfd, buf, buflen) < 0)
+			    detach_client(cfd);
+		    } else
+#endif /* RTCM104_SERVICE */
+		    {
+			if (subscribers[cfd].device)
+			    subscribers[cfd].device->poll_times[cfd] = timestamp();
+			if (handle_gpsd_request(cfd, buf, buflen) < 0)
+			    detach_client(cfd);
+		    }
+
+
 		}
 	    } else if (subscribers[cfd].device == NULL && timestamp() - subscribers[cfd].active > ASSIGNMENT_TIMEOUT) {
 		gpsd_report(1, "client timed out before assignment request.\n");
