@@ -14,16 +14,16 @@
 extern char *strtok_r(char *, const char *, char **);
 #endif /* S_SPLINT_S */
 
-#ifdef __UNUSED__
 /* 
  * check the environment to determine proper GPS units
  *
  * clients should only call this if no user preference on the command line or
  * Xresources
  *
- * return 0 - Use miles/feet
- *        1 - Use knots/feet
- *        2 - Use km/meters
+ * return imperial    - Use miles/feet
+ *        nautical    - Use knots/feet
+ *        metric      - Use km/meters
+ *        unspecified - use compiled default
  * 
  * In order check these environment vars:
  *    GPSD_UNITS one of: 
@@ -43,36 +43,35 @@ extern char *strtok_r(char *, const char *, char **);
  *
  * if none found then return compiled in default
  */
-int gpsd_units(void)
+enum unit gpsd_units(void)
 {
 	char *envu = NULL;
 
- 	if ((envu = getenv("GPSD_UNITS")) && *envu) {
-		if (strcasecmp(envu, "imperial")) {
-			return 0;
+ 	if ((envu = getenv("GPSD_UNITS")) != NULL && *envu != '\0') {
+		if (0 == strcasecmp(envu, "imperial")) {
+			return imperial;
 		}
-		if (strcasecmp(envu, "nautical")) {
-			return 1;
+		if (0 == strcasecmp(envu, "nautical")) {
+			return nautical;
 		}
-		if (strcasecmp(envu, "metric")) {
-			return 2;
+		if (0 == strcasecmp(envu, "metric")) {
+			return metric;
 		}
 		/* unrecognized, ignore it */
 	}
- 	if (((envu = getenv("LC_MEASUREMENT")) && *envu) 
- 	    || ((envu = getenv("LANG")) && *envu)) {
-		if (   strstr(envu, "_US") 
-		    || strcasecmp(envu, "C")
-		    || strcasecmp(envu, "POSIX")) {
-			return 0;
+ 	if (((envu = getenv("LC_MEASUREMENT")) != NULL && *envu != '\0') 
+ 	    || ((envu = getenv("LANG")) != NULL && *envu != '\0')) {
+		if (strcasecmp(envu, "en_US")==0 
+		    || strcasecmp(envu, "C")==0
+		    || strcasecmp(envu, "POSIX")==0) {
+			return imperial;
 		}
 		/* Other, must be metric */
-		return 2;
+		return metric;
 	}
 	/* TODO: allow a compile time default here */
-	return 0;
+	return unspecified;
 }
-#endif /* __UNUSED__ */
 
 struct gps_data_t *gps_open(const char *host, const char *port)
 /* open a connection to a gpsd daemon */
@@ -158,9 +157,13 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    break;
 		case 'C':
 		    if (sp[2] == '?')
-			gpsdata->cycle = 0;
-		    else
-			(void)sscanf(sp, "C=%u", &gpsdata->cycle);
+			gpsdata->mincycle = gpsdata->cycle = 0;
+		    else {
+			if (sscanf(sp, "C=%lf %lf", 
+				     &gpsdata->cycle,
+				   &gpsdata->mincycle) < 2)
+			    gpsdata->mincycle = gpsdata->cycle;
+		    }
 		    break;
 		case 'D':
 		    if (sp[2] == '?') 
@@ -253,7 +256,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			char eph[20], epv[20], track[20],speed[20], climb[20];
 			char epd[20], eps[20], epc[20];
 			int st = sscanf(sp+2, 
-			       "%6s %lf %lf %lf %lf %s %s %s %s %s %s %s %s %s",
+			       "%8s %lf %lf %lf %lf %s %s %s %s %s %s %s %s %s",
 				tag, &nf.time, &nf.ept, 
 				&nf.latitude, &nf.longitude,
 			        alt, eph, epv, track, speed, climb,
@@ -285,7 +288,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 				gpsdata->set |= SPEEDERR_SET;
 			    if (isnan(nf.epc)==0)
 				gpsdata->set |= CLIMBERR_SET;
-
+			    nf.pitch = nf.roll = nf.dip = NAN;
 			    gpsdata->fix = nf;
 			    (void)strcpy(gpsdata->tag, tag);
 			    gpsdata->set = TIME_SET|TIMERR_SET|LATLON_SET|MODE_SET;
