@@ -11,20 +11,30 @@
 #include <errno.h>
 #include <fcntl.h>
 
-#include "config.h"
+#include "gpsd.h"
+
 #if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
 #include <pthread.h>
 #endif
 
-#include "gpsd.h"
 
 int gpsd_switch_driver(struct gps_device_t *session, char* typename)
 {
     struct gps_type_t **dp;
+
+    /* make it idempotent */
+    if (session->device_type != NULL && 
+		strcmp(session->device_type->typename, typename) == 0)
+	return 0;
+
     /*@ -compmempass @*/
     for (dp = gpsd_drivers; *dp; dp++)
 	if (strcmp((*dp)->typename, typename) == 0) {
 	    gpsd_report(3, "Selecting %s driver...\n", (*dp)->typename);
+            if (session->saved_baud == -1)
+                session->saved_baud = (int)cfgetispeed(&session->ttyset);
+	    if (session->device_type != NULL && session->device_type->wrapup != NULL)
+		session->device_type->wrapup(session);
 	    /*@i@*/session->device_type = *dp;
 	    if (session->device_type->initializer)
 		session->device_type->initializer(session);
@@ -60,6 +70,9 @@ void gpsd_init(struct gps_device_t *session, struct gps_context_t *context, char
 
     /* necessary in case we start reading in the middle of a GPGSV sequence */
     gpsd_zero_satellites(&session->gpsdata);
+
+    /* initialize things for the packet parser */
+    packet_reset(session);
 }
 
 void gpsd_deactivate(struct gps_device_t *session)
@@ -67,7 +80,6 @@ void gpsd_deactivate(struct gps_device_t *session)
 {
     gpsd_report(1, "closing GPS=%s (%d)\n", 
 		session->gpsdata.gps_device, session->gpsdata.gps_fd);
-    (void)gpsd_close(session);
 #ifdef NTPSHM_ENABLE
     (void)ntpshm_free(session->context, session->shmTime);
     session->shmTime = -1;
@@ -78,6 +90,7 @@ void gpsd_deactivate(struct gps_device_t *session)
 #endif /* NTPSHM_ENABLE */
     if (session->device_type != NULL && session->device_type->wrapup != NULL)
 	session->device_type->wrapup(session);
+    (void)gpsd_close(session);
 }
 
 #if defined(PPS_ENABLE) && defined(TIOCMIWAIT)
@@ -150,17 +163,18 @@ int gpsd_activate(struct gps_device_t *session)
     else {
 	session->gpsdata.online = timestamp();
 #ifdef SIRFII_ENABLE
-	session->sirf.satcounter = 0;
+	session->driver.sirf.satcounter = 0;
 #endif /* SIRFII_ENABLE */
-	session->counter = 0;
+	session->char_counter = 0;
+	session->retry_counter = 0;
 	gpsd_report(1, "gpsd_activate: opened GPS (%d)\n", session->gpsdata.gps_fd);
 	// session->gpsdata.online = 0;
 	session->gpsdata.fix.mode = MODE_NOT_SEEN;
 	session->gpsdata.status = STATUS_NO_FIX;
 	session->gpsdata.fix.track = NAN;
+	session->gpsdata.separation = NAN;
 #ifdef BINARY_ENABLE
 	session->mag_var = NAN;
-	session->gpsdata.separation = NAN;
 #endif /* BINARY_ENABLE */
 
 #ifdef NTPSHM_ENABLE
@@ -295,15 +309,15 @@ static void gpsd_binary_satellite_dump(struct gps_device_t *session,
     }
 
 #ifdef ZODIAC_ENABLE
-    if (session->packet_type == ZODIAC_PACKET && session->zodiac.Zs[0] != 0) {
+    if (session->packet_type == ZODIAC_PACKET && session->driver.zodiac.Zs[0] != 0) {
 	bufp += strlen(bufp);
 	bufp2 = bufp;
 	strcpy(bufp, "$PRWIZCH");
 	for (i = 0; i < ZODIAC_CHANNELS; i++) {
 	    len -= snprintf(bufp+strlen(bufp), len,
 			  ",%02u,%X", 
-			    session->zodiac.Zs[i], 
-			    session->zodiac.Zv[i] & 0x0f);
+			    session->driver.zodiac.Zs[i], 
+			    session->driver.zodiac.Zv[i] & 0x0f);
 	}
 	nmea_add_checksum(bufp2);
     }
@@ -319,7 +333,7 @@ static void gpsd_binary_quality_dump(struct gps_device_t *session,
     (void)snprintf(bufp, len-strlen(bufp),
 		   "$GPGSA,%c,%d,", 'A', session->gpsdata.fix.mode);
     j = 0;
-    for (i = 0; i < NMEA_CHANNELS; i++) {
+    for (i = 0; i < session->device_type->channels; i++) {
 	if (session->gpsdata.used[i]) {
 	    bufp += strlen(bufp);
 	    (void)snprintf(bufp, len-strlen(bufp),
@@ -327,7 +341,7 @@ static void gpsd_binary_quality_dump(struct gps_device_t *session,
 	    j++;
 	}
     }
-    for (i = j; i < NMEA_CHANNELS; i++) {
+    for (i = j; i < session->device_type->channels; i++) {
 	bufp += strlen(bufp);
 	(void)strcpy(bufp, ",");
     }
@@ -354,7 +368,6 @@ static void gpsd_binary_quality_dump(struct gps_device_t *session,
 	    ZEROIZE(session->gpsdata.fix.epv), 
 	    ZEROIZE(session->gpsdata.epe));
         nmea_add_checksum(bufp);
-	session->gpsdata.seen_sentences |= PGRME;
      }
 #undef ZEROIZE
 }
@@ -378,9 +391,8 @@ static void apply_error_model(struct gps_device_t *session)
      * Some drivers set the position-error fields.  Only the Zodiacs 
      * report speed error.  Nobody reports track error or climb error.
      */
-    /* only used if the GPS doesn't report estimated position error itself */
-#define UERE_NO_DGPS	8.0	/* meters */
-#define UERE_WITH_DGPS	2.0	/* meters */
+#define UERE_NO_DGPS	8.0	/* meters, 95% confidence */
+#define UERE_WITH_DGPS	2.0	/* meters, 95% confidence */
     double uere = (session->gpsdata.status == STATUS_DGPS_FIX ? UERE_WITH_DGPS : UERE_NO_DGPS);
 
     session->gpsdata.fix.ept = 0.005;
@@ -459,43 +471,79 @@ static void apply_error_model(struct gps_device_t *session)
 gps_mask_t gpsd_poll(struct gps_device_t *session)
 /* update the stuff in the scoreboard structure */
 {
-    ssize_t packet_length;
+    ssize_t newdata;
 
     if (session->inbuflen==0)
 	session->gpsdata.d_xmit_time = timestamp();
 
     /* can we get a full packet from the device? */
-    if (session->device_type)
-	packet_length = session->device_type->get_packet(session);
-    else {
-	packet_length = packet_get(session);
+    if (session->device_type) {
+	newdata = session->device_type->get_packet(session);
+	session->gpsdata.d_xmit_time = timestamp();
+    } else {
+	newdata = packet_get(session);
+	session->gpsdata.d_xmit_time = timestamp();
+	gpsd_report(3, 
+		    "packet sniff finds type %d\n", 
+		    session->packet_type);
 	if (session->packet_type != BAD_PACKET) {
-	    gpsd_report(3, 
-			"packet sniff finds type %d\n", 
-			session->packet_type);
-	    if (session->packet_type == SIRF_PACKET)
+	    switch (session->packet_type) {
+#ifdef SIRFII_ENABLE
+	    case SIRF_PACKET:
 		(void)gpsd_switch_driver(session, "SiRF-II binary");
-	    else if (session->packet_type == TSIP_PACKET)
+		break;
+#endif /* SIRFII_ENABLE */
+#ifdef TSIP_ENABLE
+	    case TSIP_PACKET:
 		(void)gpsd_switch_driver(session, "Trimble TSIP");
-	    else if (session->packet_type == NMEA_PACKET)
+		break;
+#endif /* TSIP_ENABLE */
+#ifdef NMEA_ENABLE
+	    case NMEA_PACKET:
 		(void)gpsd_switch_driver(session, "Generic NMEA");
-	    else if (session->packet_type == ZODIAC_PACKET)
+		break;
+#endif /* NMEA_ENABLE */
+#ifdef ZODIAC_ENABLE
+	    case ZODIAC_PACKET:
 		(void)gpsd_switch_driver(session, "Zodiac binary");
-	    session->gpsdata.d_xmit_time = timestamp();
+		break;
+#endif /* ZODIAC_ENABLE */
+#ifdef EVERMORE_ENABLE
+	    case EVERMORE_PACKET:
+		(void)gpsd_switch_driver(session, "EverMore binary");
+		break;
+#endif /* EVERMORE_ENABLE */
+#ifdef ITALK_ENABLE
+	    case ITALK_PACKET:
+		(void)gpsd_switch_driver(session, "iTalk binary");
+		break;
+#endif /* ITALK_ENABLE */
+#ifdef RTCM104_ENABLE
+	    case RTCM_PACKET:
+		(void)gpsd_switch_driver(session, "RTCM104");
+		break;
+#endif /* RTCM104_ENABLE */
+	    }
 	} else if (!gpsd_next_hunt_setting(session))
 	    return ERROR_SET;
     }
 
     /* update the scoreboard structure from the GPS */
-    gpsd_report(7, "GPS sent %d characters\n", packet_length);
-    if (packet_length == BAD_PACKET)
+    gpsd_report(7, "GPS sent %d new characters\n", newdata);
+    if (newdata == -1)	{		/* read error */
+	session->gpsdata.online = 0;
 	return 0;
-    else if (packet_length == 0) {
+    } else if (newdata == 0) {		/* no new data */
 	if (session->device_type != NULL && timestamp()>session->gpsdata.online+session->device_type->cycle+1){
+		gpsd_report(3, "GPS is offline (%lf sec since data)\n", 
+			timestamp() - session->gpsdata.online);
 	    session->gpsdata.online = 0;
 	    return 0;
 	} else
 	    return ONLINE_SET;
+    } else if (session->outbuflen == 0) {   /* got new data, but no packet */
+	    gpsd_report(8, "New data, not yet a packet\n");
+	    return ONLINE_SET;
     } else {
 	gps_mask_t received, dopmask = 0;
 	session->gpsdata.online = timestamp();
@@ -504,14 +552,17 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	if (session->gpsdata.raw_hook)
 	    session->gpsdata.raw_hook(&session->gpsdata, 
 				      (char *)session->outbuffer,
-				      (size_t)packet_length, 2);
+				      (size_t)session->outbuflen, 2);
 	/*@ -nullstate @*/
 	session->gpsdata.sentence_time = NAN;
 	session->gpsdata.sentence_length = session->outbuflen;
 	session->gpsdata.d_recv_time = timestamp();
 
 	/* Get data from current packet into the newdata structure */
-	received = session->device_type->parse_packet(session);
+	if (session->device_type != NULL && session->device_type->parse_packet!=NULL)
+	    received = session->device_type->parse_packet(session);
+	else
+	    received = 0;	/* it was all done in the packet getter */
 
 	/* Clear fix data at start of cycle */
 	if ((received & CYCLE_START_SET)!=0) {
@@ -530,8 +581,7 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	gps_merge_fix(&session->gpsdata.fix, received, &session->gpsdata.newdata);
 	session->gpsdata.set = ONLINE_SET | dopmask | received;
 
-	/* count all packets and good fixes */
-	session->counter++;
+	/* count good fixes */
 	if (session->gpsdata.status > STATUS_NO_FIX) 
 	    session->context->fixcnt++;
 
@@ -555,6 +605,13 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 	    char buf2[MAX_PACKET_LENGTH*3+2];
 
 	    buf2[0] = '\0';
+#ifdef RTCM104_ENABLE
+	    if ((session->gpsdata.set & RTCM_SET) != 0)
+		rtcm_dump(session, 
+			  buf2+strlen(buf2), 
+			  (sizeof(buf2)-strlen(buf2)));
+	    else
+#endif /* RTCM104_ENABLE */
 	    if ((session->gpsdata.set & LATLON_SET) != 0)
 		gpsd_binary_fix_dump(session, 
 				     buf2+strlen(buf2), 
@@ -596,3 +653,122 @@ void gpsd_zero_satellites(/*@out@*/struct gps_data_t *out)
     out->satellites = 0;
 }
 
+char /*@ observer @*/ *gpsd_hexdump(const void *binbuf, size_t binbuflen)
+{
+    static char hexbuf[MAX_PACKET_LENGTH*2+1];
+    size_t i;
+    size_t len = (size_t)((binbuflen > MAX_PACKET_LENGTH) ? MAX_PACKET_LENGTH : binbuflen);
+    char *ibuf = (char *)binbuf;
+    memset(hexbuf, 0, sizeof(hexbuf));
+
+    for (i = 0; i < len; i++) {
+	(void)snprintf(hexbuf + (2 * i), 3, "%02x", (unsigned int)(ibuf[i]&0xff));
+    }
+    return hexbuf;
+}
+
+#ifdef BINARY_ENABLE
+/*@ -usedef @*/
+void gpsd_interpret_subframe(struct gps_device_t *session,unsigned int words[])
+/* extract leap-second from RTCM-104 subframe data */
+{
+    /*
+     * Heavy black magic begins here!
+     *
+     * A description of how to decode these bits is at
+     * <http://home-2.worldonline.nl/~samsvl/nav2eu.htm>
+     *
+     * We're after subframe 4 page 18 word 9, the leap year correction.
+     * We assume that the chip is presenting clean data that has been
+     * parity-checked.
+     *
+     * To date this code has been tested only on SiRFs.  It's in the
+     * core because other chipsets reporting only GPS time but with 
+     * the capability to read subframe data may want it.
+     */
+    int i;
+    unsigned int pageid, subframe, leap;
+    gpsd_report(4, 
+		"50B (raw): %08x %08x %08x %08x %08x %08x %08x %08x %08x %08x\n", 
+		words[0], words[1], words[2], words[3], words[4], 
+		words[5], words[6], words[7], words[8], words[9]);
+    /*
+     * Mask off the high 2 bits and shift out the 6 parity bits.
+     * Once we've filtered, we can ignore the TEL and HOW words.
+     * We don't need to check parity here, the SiRF chipset does
+     * that and throws a subframe error if the parity is wrong.
+     */
+    for (i = 0; i < 10; i++)
+	words[i] = (words[i]  & 0x3fffffff) >> 6;
+    /*
+     * "First, throw away everything that doesn't start with 8b or
+     * 74. more correctly the first byte should be 10001011. If
+     * it's 01110100, then you have a subframe with inverted
+     * polarity and each byte needs to be xored against 0xff to
+     * remove the inversion."
+     */
+    words[0] &= 0xff0000;
+    if (words[0] != 0x8b0000 && words[0] != 0x740000)
+	return;
+    if (words[0] == 0x740000)
+	for (i = 1; i < 10; i++)
+	    words[i] ^= 0xffffff;
+    /*
+     * The subframe ID is in the Hand Over Word (page 80) 
+     */
+    subframe = ((words[1] >> 2) & 0x07);
+    /* we're not interested in anything but subframe 4 */
+    if (subframe != 4)
+	return;
+    /*
+     * Pages 66-76a,80 of ICD-GPS-200 are the subframe structures.
+     * Subframe 4 page 18 is on page 74.
+     * See page 105 for the mapping between magic SVIDs and pages.
+     */
+    pageid = (words[2] & 0x3F0000) >> 16;
+    gpsd_report(2, "Subframe 4 SVID is %d\n", pageid);
+    if (pageid == 56) {	/* magic SVID for page 18 */
+	/* once we've filtered, we can ignore the TEL and HOW words */
+	gpsd_report(2, "50B: SF=%d %06x %06x %06x %06x %06x %06x %06x %06x\n", 
+		    subframe,
+		    words[2], words[3], words[4], words[5], 
+		    words[6], words[7], words[8], words[9]);
+	leap = (words[8] & 0xff0000) >> 16;
+	/*
+	 * On SiRFs, there appears to be some bizarre bug that
+	 * randomly causes this field to come out two's-complemented.
+	 * This could very well be a general problem; work around it.
+	 * At the current expected rate of issuing leap-seconds this
+	 * kluge won't bite until about 2070, by which time the
+	 * vendors had better have fixed their damn firmware...
+	 *
+	 * Carl: ...I am unsure, and suggest you
+	 * experiment.  The D30 bit is in bit 30 of the 32-bit
+	 * word (next to MSB), and should signal an inverted
+	 * value when it is one coming over the air.  But if
+	 * the bit is set and the word decodes right without
+	 * inversion, then we properly caught it.  Cases where
+	 * you see subframe 6 rather than 1 means we should
+	 * have done the inversion but we did not.  Some other
+	 * things you can watch for: in any subframe, the
+	 * second word (HOW word) should have last 2 parity
+	 * bits 00 -- there are bits within the rest of the
+	 * word that are set as required to ensure that.  The
+	 * same goes for word 10.  That means that both words
+	 * 1 and 3 (the words that immediately follow words 10
+	 * and 2, respectively) should always be uninverted.
+	 * In these cases, the D29 and D30 from the previous
+	 * words, found in the two MSBs of the word, should
+	 * show 00 -- if they don't then you may find an
+	 * unintended inversion due to noise on the data link.
+	 */
+	if (leap > 128)
+	    leap ^= 0xff;
+	gpsd_report(2, "leap-seconds is %d\n", leap);
+	session->context->leap_seconds = (int)leap;
+	session->context->valid |= LEAP_SECOND_VALID;
+    }
+}
+/*@ +usedef @*/
+#endif /* BINARY_ENABLE */
+
