@@ -28,43 +28,23 @@ distinguish them from baud barf.
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-#include <sys/time.h>
-#include <sys/ioctl.h>
+#include <sys/types.h>
 #include "config.h"
 #include "gpsd.h"
 
-#ifdef TESTMAIN
-#include <stdarg.h>
-
-static int verbose = 0;
-static int waiting;
-
-void gpsd_report(int errlevel, const char *fmt, ... )
-/* assemble command in printf(3) style, use stderr or syslog */
-{
-    if (errlevel <= verbose) {
-	char buf[BUFSIZ];
-	va_list ap;
-
-	buf[0] = '\0';
-	va_start(ap, fmt) ;
-	(void)vsnprintf(buf + strlen(buf), sizeof(buf)-strlen(buf), fmt, ap);
-	va_end(ap);
-
-	(void)fputs(buf, stderr);
-    }
-}
-#endif /* TESTMAIN */
-
 /* 
- * The packet-recognition state machine.  It doesn't do checksums,
- * caller is responsible for that part.  It can be fooled by garbage
+ * The packet-recognition state machine.  It can be fooled by garbage
  * that looks like the head of a binary packet followed by a NMEA
  * packet; in that case it won't reset until it notices that the
  * binary trailer is not where it should be, and the NMEA packet will
  * be lost.  The reverse scenario is not possible because none of the
  * binary leader character can occur in an NMEA packet.  Caller should
  * consume a packet when it sees one of the *_RECOGNIZED states.
+ * It's good practice to follow the _RECOGNIZED transition with one
+ * that recognizes a leader of the same packet type rather than
+ * dropping back to ground state -- this for example will prevent
+ * the state machine from hopping between recognizing TSIP and
+ * EverMore packets that both start with a DLE.
  *
  * Error handling is brutally simple; any time we see an unexpected
  * character, go to GROUND_STATE and reset the machine (except that a
@@ -78,93 +58,86 @@ void gpsd_report(int errlevel, const char *fmt, ... )
  *      GP -- Global Positioning System.
  *      II -- Integrated Instrumentation (Raytheon's SeaTalk system).
  *	IN -- Integrated Navigation (Garmin uses this).
+ *
  */
 
 enum {
-   GROUND_STATE,	/* we don't know what packet type to expect */
-   NMEA_DOLLAR,		/* we've seen first character of NMEA leader */
-
-   NMEA_PUB_LEAD,	/* seen second character of NMEA G leader */
-   NMEA_LEADER_END,	/* seen end char of NMEA leader, in body */
-   NMEA_CR,	   	/* seen terminating \r of NMEA packet */
-   NMEA_RECOGNIZED,	/* saw trailing \n of NMEA packet */
-
-   SEATALK_LEAD_1,	/* SeaTalk/Garmin packet leader 'I' */
-
-#ifdef TRIPMATE_ENABLE
-   ASTRAL_1,		/* ASTRAL leader A */
-   ASTRAL_2,	 	/* ASTRAL leader S */
-   ASTRAL_3,		/* ASTRAL leader T */
-   ASTRAL_4,		/* ASTRAL leader R */
-   ASTRAL_5,		/* ASTRAL leader A */
-#endif /* TRIPMATE_ENABLE */
-
-#ifdef EARTHMATE_ENABLE
-   EARTHA_1,		/* EARTHA leader E */
-   EARTHA_2,		/* EARTHA leader A */
-   EARTHA_3,		/* EARTHA leader R */
-   EARTHA_4,		/* EARTHA leader T */
-   EARTHA_5,		/* EARTHA leader H */
-#endif /* EARTHMATE_ENABLE */
-
-   SIRF_LEADER_1,	/* we've seen first character of SiRF leader */
-   SIRF_LEADER_2,	/* seen second character of SiRF leader */
-   SIRF_ACK_LEAD_1,	/* seen A of possible SiRF Ack */
-   SIRF_ACK_LEAD_2,	/* seen c of possible SiRF Ack */
-   SIRF_LENGTH_1,	/* seen first byte of SiRF length */
-   SIRF_PAYLOAD,	/* we're in a SiRF payload part */
-   SIRF_DELIVERED,	/* saw last byte of SiRF payload/checksum */
-   SIRF_TRAILER_1,	/* saw first byte of SiRF trailer */ 
-   SIRF_RECOGNIZED,	/* saw second byte of SiRF trailer */
-
-   TSIP_LEADER,		/* we've seen the TSIP leader (DLE) */
-   TSIP_PAYLOAD,	/* we're in TSIP payload */
-   TSIP_DLE,		/* we've seen a DLE in TSIP payload */
-   TSIP_RECOGNIZED,	/* found end of the TSIP packet */
-
-   ZODIAC_EXPECTED,	/* expecting Zodiac packet */
-   ZODIAC_LEADER_1,	/* saw leading 0xff */
-   ZODIAC_LEADER_2,	/* saw leading 0x81 */
-   ZODIAC_ID_1, 	/* saw first byte of ID */
-   ZODIAC_ID_2, 	/* saw second byte of ID */
-   ZODIAC_LENGTH_1,	/* saw first byte of Zodiac packet length */
-   ZODIAC_LENGTH_2,	/* saw second byte of Zodiac packet length */
-   ZODIAC_FLAGS_1, 	/* saw first byte of FLAGS */
-   ZODIAC_FLAGS_2, 	/* saw second byte of FLAGS */
-   ZODIAC_HSUM_1, 	/* saw first byte of Header sum */
-   ZODIAC_PAYLOAD,	/* we're in a Zodiac payload */
-   ZODIAC_RECOGNIZED,	/* found end of the Zodiac packet */
+#include "packet_states.h"
 };
 
-static void nexstate(struct gps_device_t *session, unsigned char c)
+static void nextstate(struct gps_device_t *session, unsigned char c)
 {
+#ifdef RTCM104_ENABLE
+    enum isgpsstat_t	isgpsstat;    
+#endif /* RTCM104_ENABLE */
 /*@ +charint */
     switch(session->packet_state)
     {
     case GROUND_STATE:
-	if (c == '$')
+#ifdef NMEA_ENABLE
+	if (c == '$') {
 	    session->packet_state = NMEA_DOLLAR;
+	    break;
+	}
+#endif /* NMEA_ENABLE */
+
+#ifdef TNT_ENABLE
+        if (c == '@') {
+	    session->packet_state = TNT_LEADER;
+	    break;
+	}
+#endif
 #ifdef SIRFII_ENABLE
-        else if (c == 0xa0)
+        if (c == 0xa0) {
 	    session->packet_state = SIRF_LEADER_1;
+	    break;
+	}
 #endif /* SIRFII_ENABLE */
-#ifdef TSIP_ENABLE
-        else if (c == 0x10)
-	    session->packet_state = TSIP_LEADER;
-#endif /* TSIP_ENABLE */
+#if defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE)
+        if (c == 0x10) {
+	    session->packet_state = DLE_LEADER;
+	    break;
+	}
+#endif /* defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE) */
 #ifdef TRIPMATE_ENABLE
-        else if (c == 'A')
+        if (c == 'A') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = ASTRAL_1;
+	    break;
+	}
 #endif /* TRIPMATE_ENABLE */
 #ifdef EARTHMATE_ENABLE
-        else if (c == 'E')
+        if (c == 'E') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = EARTHA_1;
+	    break;
+	}
 #endif /* EARTHMATE_ENABLE */
 #ifdef ZODIAC_ENABLE
-	else if (c == 0xff)
+	if (c == 0xff) {
 	    session->packet_state = ZODIAC_LEADER_1;
+	    break;
+	}
 #endif /* ZODIAC_ENABLE */
+#ifdef ITALK_ENABLE
+	if (c == '<') {
+	    session->packet_state = ITALK_LEADER_1;
+	    break;
+	}
+#endif /* ITALK_ENABLE */
+#ifdef RTCM104_ENABLE
+	if (rtcm_decode(session, c) == ISGPS_SYNC) {
+	    session->packet_state = RTCM_SYNC_STATE;
+	    break;
+	}
+#endif /* RTCM104_ENABLE */
 	break;
+	/*@ +casebreak @*/
+#ifdef NMEA_ENABLE
     case NMEA_DOLLAR:
 	if (c == 'G')
 	    session->packet_state = NMEA_PUB_LEAD;
@@ -183,6 +156,11 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	else
 	    session->packet_state = GROUND_STATE;
 	break;
+#ifdef TNT_ENABLE
+    case TNT_LEADER:
+          session->packet_state = NMEA_LEADER_END;
+        break;
+#endif
     case NMEA_LEADER_END:
 	if (c == '\r')
 	    session->packet_state = NMEA_CR;
@@ -215,65 +193,95 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	break;
 #ifdef TRIPMATE_ENABLE
     case ASTRAL_1:
-	if (c == 'S')
+	if (c == 'S') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = ASTRAL_2;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
 	break;
     case ASTRAL_2:
-	if (c == 'T')
+	if (c == 'T') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = ASTRAL_3;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
 	break;
     case ASTRAL_3:
-	if (c == 'R')
+	if (c == 'R') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = ASTRAL_5;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
 	break;
     case ASTRAL_4:
-	if (c == 'A')
+	if (c == 'A') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = ASTRAL_2;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
 	break;
     case ASTRAL_5:
-	if (c == 'L')
+	if (c == 'L') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = NMEA_RECOGNIZED;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
-	break; 
+	break;
 #endif /* TRIPMATE_ENABLE */
 #ifdef EARTHMATE_ENABLE
     case EARTHA_1:
-	if (c == 'A')
+	if (c == 'A') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = EARTHA_2;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
 	break;
     case EARTHA_2:
-	if (c == 'R')
+	if (c == 'R') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = EARTHA_3;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
 	break;
     case EARTHA_3:
-	if (c == 'T')
+	if (c == 'T') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = EARTHA_4;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
 	break;
     case EARTHA_4:
-	if (c == 'H')
+	if (c == 'H') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = EARTHA_5;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
 	break;
     case EARTHA_5:
-	if (c == 'A')
+	if (c == 'A') {
+#ifdef RTCM104_ENABLE
+	    (void)rtcm_decode(session, c);
+#endif /* RTCM104_ENABLE */
 	    session->packet_state = NMEA_RECOGNIZED;
-	else
+	} else
 	    session->packet_state = GROUND_STATE;
 	break; 
 #endif /* EARTHMATE_ENABLE */
@@ -289,6 +297,7 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	else
 	    session->packet_state = GROUND_STATE;
 	break;
+#endif /* NMEA_ENABLE */
 #ifdef SIRFII_ENABLE
     case SIRF_LEADER_1:
 	if (c == 0xa2)
@@ -330,38 +339,22 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	    session->packet_state = GROUND_STATE;
 	break;
 #endif /* SIRFII_ENABLE */
+#if defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE)
+    case DLE_LEADER:
+#ifdef EVERMORE_ENABLE
+	if (c == 0x02)
+	    session->packet_state = EVERMORE_LEADER_2;
+	else
+#endif /* EVERMORE_ENABLE */
 #ifdef TSIP_ENABLE
-    case TSIP_LEADER:
-	if (c < 0x13)
-	    session->packet_state = GROUND_STATE;
-	else
+	/* check last because there's no checksum */
+	if (c >= 0x13)
 	    session->packet_state = TSIP_PAYLOAD;
-	break;
-    case TSIP_PAYLOAD:
-	if (c == 0x10)
-	    session->packet_state = TSIP_DLE;
-	break;
-    case TSIP_DLE:
-	switch (c)
-	{
-	case 0x03:
-	    session->packet_state = TSIP_RECOGNIZED;
-	    break;
-	case 0x10:
-	    session->packet_state = TSIP_PAYLOAD;
-	    break;
-	default:
-	    session->packet_state = GROUND_STATE;
-	    break;
-	}
-	break;
-    case TSIP_RECOGNIZED:
-        if (c == 0x10)
-	    session->packet_state = TSIP_LEADER;
 	else
-	    session->packet_state = GROUND_STATE;
-	break;
 #endif /* TSIP_ENABLE */
+	    session->packet_state = GROUND_STATE;
+	break;
+#endif /* defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE) */
 #ifdef ZODIAC_ENABLE
     case ZODIAC_EXPECTED:
     case ZODIAC_RECOGNIZED:
@@ -430,27 +423,140 @@ static void nexstate(struct gps_device_t *session, unsigned char c)
 	    session->packet_state = ZODIAC_RECOGNIZED;
 	break;
 #endif /* ZODIAC_ENABLE */
+#ifdef EVERMORE_ENABLE
+    case EVERMORE_LEADER_1:
+	if (c == 0x02)
+	    session->packet_state = EVERMORE_LEADER_2;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
+    case EVERMORE_LEADER_2:
+	session->packet_length = (size_t)c;
+	if (c == 0x10)
+	    session->packet_state = EVERMORE_PAYLOAD_DLE;
+	else
+	    session->packet_state = EVERMORE_PAYLOAD;
+	break;
+    case EVERMORE_PAYLOAD:
+	if (c == 0x10)
+	    session->packet_state = EVERMORE_PAYLOAD_DLE;
+	else if (--session->packet_length == 0)
+	    session->packet_state = GROUND_STATE;
+	break;
+    case EVERMORE_PAYLOAD_DLE:
+        switch (c) {
+           case 0x10: session->packet_state = EVERMORE_PAYLOAD; break;
+           case 0x03: session->packet_state = EVERMORE_RECOGNIZED; break;
+           default: session->packet_state = GROUND_STATE;
+        }
+    break;
+    case EVERMORE_RECOGNIZED:
+        if (c == 0x10)
+	    session->packet_state = EVERMORE_LEADER_1;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
+#endif /* EVERMORE_ENABLE */
+#ifdef ITALK_ENABLE
+    case ITALK_LEADER_1:
+        if (c == '*')
+	    session->packet_state = ITALK_LEADER_2;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
+    case ITALK_LEADER_2:
+	session->packet_length = (size_t)(c << 8);
+	session->packet_state = ITALK_LENGTH_1;
+	break;
+    case ITALK_LENGTH_1:
+	session->packet_length += c + 1;
+	session->packet_length *= 2;	/* count is in words */
+	session->packet_state = ITALK_LENGTH_2;
+	break;
+    case ITALK_LENGTH_2:
+	if (--session->packet_length == 0)
+	    session->packet_state = ITALK_DELIVERED;
+	break;
+    case ITALK_DELIVERED:
+	if (c == '>')
+	    session->packet_state = ITALK_TRAILER_1;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
+    case ITALK_TRAILER_1:
+	if (c == 0xb3)
+	    session->packet_state = ITALK_RECOGNIZED;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
+    case ITALK_RECOGNIZED:
+        if (c == 0xa0)
+	    session->packet_state = ITALK_LEADER_1;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
+#endif /* ITALK_ENABLE */
+#ifdef TSIP_ENABLE
+    case TSIP_LEADER:
+	if (c >= 0x13)
+	    session->packet_state = TSIP_PAYLOAD;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
+    case TSIP_PAYLOAD:
+	if (c == 0x10)
+	    session->packet_state = TSIP_DLE;
+	break;
+    case TSIP_DLE:
+	switch (c)
+	{
+	case 0x03:
+	    session->packet_state = TSIP_RECOGNIZED;
+	    break;
+	case 0x10:
+	    session->packet_state = TSIP_PAYLOAD;
+	    break;
+	default:
+	    session->packet_state = GROUND_STATE;
+	    break;
+	}
+	break;
+    case TSIP_RECOGNIZED:
+        if (c == 0x10)
+	    /*
+	     * Don't go to TSIP_LEADER state -- TSIP packets aren't
+	     * checksummed, so false positives are easy.  We might be
+	     * looking at another DLE-stuffed protocol like EverMore.
+	     */
+	    session->packet_state = DLE_LEADER;
+	else
+	    session->packet_state = GROUND_STATE;
+	break;
+#endif /* TSIP_ENABLE */
+#ifdef RTCM104_ENABLE
+    case RTCM_SYNC_STATE:
+    case RTCM_SKIP_STATE:
+	isgpsstat = rtcm_decode(session, c);
+	if (isgpsstat == ISGPS_MESSAGE) {
+	    session->packet_state = RTCM_RECOGNIZED;
+	    break;
+	} else if (isgpsstat == ISGPS_NO_SYNC)
+	    session->packet_state = GROUND_STATE;
+	break;
+
+    case RTCM_RECOGNIZED:
+	if (rtcm_decode(session, c) == ISGPS_SYNC) {
+	    session->packet_state = RTCM_SYNC_STATE;
+	    break;
+	} else
+	    session->packet_state = GROUND_STATE;
+	break;
+#endif /* RTCM104_ENABLE */
     }
 /*@ -charint */
 }
 
-#ifdef STATE_DEBUG
-static char *buffer_dump(unsigned char *base, unsigned char *end)
-/* dump the state of a specified buffer */
-{
-    static unsigned char buf[BUFSIZ];
-    unsigned char *cp, *tp = buf;
-    for (cp = base; cp < end; cp++)
-	if (isgraph(*cp)) {
-	    *tp++ = *cp;
-	}else {
-	    (void)snprintf(tp, sizeof(buf)-(tp-buf), "\\x%02x", *cp);
-	    tp += 4;
-	}
-    *tp = '\0';
-    return buf;
-}
-#endif /* STATE_DEBUG */
+#define STATE_DEBUG
 
 static void packet_accept(struct gps_device_t *session, int packet_type)
 /* packet grab succeeded, move to output buffer */
@@ -462,10 +568,9 @@ static void packet_accept(struct gps_device_t *session, int packet_type)
 	session->outbuffer[packetlen] = '\0';
 	session->packet_type = packet_type;
 #ifdef STATE_DEBUG
-	gpsd_report(6, "Packet type %d accepted %d = %s\n", 
+	gpsd_report(6, "Packet type %d accepted %d = %s\n",
 		packet_type, packetlen,
-		buffer_dump(session->outbuffer, 
-			    session->outbuffer+session->outbuflen));
+		gpsd_hexdump(session->outbuffer, session->outbuflen));
 #endif /* STATE_DEBUG */
     } else {
 	gpsd_report(1, "Rejected too long packet type %d len %d\n",
@@ -474,19 +579,18 @@ static void packet_accept(struct gps_device_t *session, int packet_type)
 }
 
 static void packet_discard(struct gps_device_t *session)
-/* shift the input buffer to discard all data up to current input pointer */ 
+/* shift the input buffer to discard all data up to current input pointer */
 {
     size_t discard = session->inbufptr - session->inbuffer;
     size_t remaining = session->inbuflen - discard;
-    session->inbufptr = memmove(session->inbuffer, 
-				session->inbufptr, 
+    session->inbufptr = memmove(session->inbuffer,
+				session->inbufptr,
 				remaining);
     session->inbuflen = remaining;
 #ifdef STATE_DEBUG
-    gpsd_report(6, "Packet discard of %d, chars remaining is %d = %s\n", 
-		discard, remaining, 
-		buffer_dump(session->inbuffer, 
-			    session->inbuffer + session->inbuflen));
+    gpsd_report(6, "Packet discard of %d, chars remaining is %d = %s\n",
+		discard, remaining,
+		gpsd_hexdump(session->inbuffer, session->inbuflen));
 #endif /* STATE_DEBUG */
 }
 
@@ -498,37 +602,26 @@ static void character_discard(struct gps_device_t *session)
 #ifdef STATE_DEBUG
     gpsd_report(6, "Character discarded, buffer %d chars = %s\n",
 		session->inbuflen,
-		buffer_dump(session->inbuffer, 
-			    session->inbuffer + session->inbuflen));
+		gpsd_hexdump(session->inbuffer, session->inbuflen));
 #endif /* STATE_DEBUG */
 }
 
+
 /* entry points begin here */
 
-ssize_t packet_get(struct gps_device_t *session)
+/* get 0-origin big-endian words relative to start of packet buffer */
+#define getword(i) (short)(session->inbuffer[2*(i)] | (session->inbuffer[2*(i)+1] << 8))
+
+
+ssize_t packet_parse(struct gps_device_t *session, size_t newdata)
 /* grab a packet; returns ether BAD_PACKET or the length */
 {
-    int newdata;
-#ifndef TESTMAIN
-    /*@ -modobserver @*/
-    newdata = (int)read(session->gpsdata.gps_fd, session->inbufptr, 
-			sizeof(session->inbuffer)-(session->inbufptr-session->inbuffer));
-    /*@ +modobserver @*/
-#else
-    newdata = waiting;
-#endif /* TESTMAIN */
-
-    if (newdata < 0 && errno != EAGAIN)
-	return BAD_PACKET;
-    else if (newdata == 0 || (newdata < 0 && errno == EAGAIN))
-	return 0;
-
 #ifdef STATE_DEBUG
-    gpsd_report(6, "Read %d chars to buffer offset %d (total %d): %s\n", 
+    gpsd_report(6, "Read %d chars to buffer offset %d (total %d): %s\n",
 		newdata,
-		session->inbufptr-session->inbuffer,
-		session->inbuflen+newdata, 
-		buffer_dump(session->inbufptr, session->inbufptr + newdata));
+		session->inbuflen,
+		session->inbuflen+newdata,
+		gpsd_hexdump(session->inbufptr, newdata));
 #endif /* STATE_DEBUG */
 
     session->outbuflen = 0;
@@ -537,13 +630,20 @@ ssize_t packet_get(struct gps_device_t *session)
 	/*@ -modobserver @*/
 	unsigned char c = *session->inbufptr++;
 	/*@ +modobserver @*/
-	nexstate(session, c);
-	if (isprint(c))
-	    gpsd_report(7, "Character '%c', new state: %d\n",c,session->packet_state);
-	else
-	    gpsd_report(7, "Character 0x%02x, new state: %d\n",c,session->packet_state);
+	char *state_table[] = {
+#include "packet_names.h"
+	};
+	nextstate(session, c);
+	gpsd_report(7, "%08ld: character '%c' [%02x], new state: %s\n",
+		    session->char_counter, 
+		    (isprint(c)?c:'.'), 
+		    c, 
+		    state_table[session->packet_state]);
+	session->char_counter++;
+
 	if (session->packet_state == GROUND_STATE) {
 	    character_discard(session);
+#ifdef NMEA_ENABLE
 	} else if (session->packet_state == NMEA_RECOGNIZED) {
 	    bool checksum_ok = true;
 	    char csum[3];
@@ -558,8 +658,12 @@ ssize_t packet_get(struct gps_device_t *session)
 	    }
 	    if (checksum_ok)
 		packet_accept(session, NMEA_PACKET);
-	    session->packet_state = GROUND_STATE;
+	    else
+		session->packet_state = GROUND_STATE;
 	    packet_discard(session);
+            break;
+#endif /* NMEA_ENABLE */
+#ifdef SIRFII_ENABLE
 	} else if (session->packet_state == SIRF_RECOGNIZED) {
 	    unsigned char *trailer = session->inbufptr-4;
 	    unsigned int checksum = (unsigned)((trailer[0] << 8) | trailer[1]);
@@ -569,18 +673,22 @@ ssize_t packet_get(struct gps_device_t *session)
 	    crc &= 0x7fff;
 	    if (checksum == crc)
 		packet_accept(session, SIRF_PACKET);
-	    session->packet_state = GROUND_STATE;
+	    else
+		session->packet_state = GROUND_STATE;
 	    packet_discard(session);
+            break;
+#endif /* SIRFII_ENABLE */
 #ifdef TSIP_ENABLE
 	} else if (session->packet_state == TSIP_RECOGNIZED) {
 	    if ((session->inbufptr - session->inbuffer) >= 4)
 		packet_accept(session, TSIP_PACKET);
-	    session->packet_state = GROUND_STATE;
+	    else
+		session->packet_state = GROUND_STATE;
 	    packet_discard(session);
+            break;
 #endif /* TSIP_ENABLE */
 #ifdef ZODIAC_ENABLE
 	} else if (session->packet_state == ZODIAC_RECOGNIZED) {
- #define getword(i) (short)(session->inbuffer[2*(i)] | (session->inbuffer[2*(i)+1] << 8))
 	    short len, n, sum;
 	    len = getword(2);
 	    for (n = sum = 0; n < len; n++)
@@ -590,17 +698,107 @@ ssize_t packet_get(struct gps_device_t *session)
 		packet_accept(session, ZODIAC_PACKET);
 	    } else {
 		gpsd_report(4,
-		    "Zodiac Data checksum 0x%hx over length %hd, expecting 0x%hx\n", 
+		    "Zodiac data checksum 0x%hx over length %hd, expecting 0x%hx\n",
 			sum, len, getword(5 + len));
+		session->packet_state = GROUND_STATE;
 	    }
-	    session->packet_state = GROUND_STATE;
 	    packet_discard(session);
-#undef getword
+            break;
 #endif /* ZODIAC_ENABLE */
-	}
-    }
+#ifdef EVERMORE_ENABLE
+	} else if (session->packet_state == EVERMORE_RECOGNIZED) {
+	    unsigned int n, crc, checksum, len;
+	    bool ok = false;
 
-    return (ssize_t)session->outbuflen;
+	    n = 0;
+	    /*@ +charint */
+	    do {
+	       if (session->inbuffer[n++] != 0x10) break;
+	       if (session->inbuffer[n++] != 0x02) break;
+	       len = session->inbuffer[n++];
+	       if (len == 0x10) {
+		  if (session->inbuffer[n++] != 0x10) break;
+	       }
+	       len -= 2;
+	       crc = 0;
+	       for (; len > 0; len--) {
+		  crc += session->inbuffer[n];
+		  if (session->inbuffer[n++] == 0x10) {
+		     if (session->inbuffer[n++] != 0x10) break;
+		  }
+	       }
+	       if (len > 0) break;
+	       checksum = session->inbuffer[n++];
+	       if (checksum == 0x10) {
+		  if (session->inbuffer[n++] != 0x10) break;
+	       }
+	       if (session->inbuffer[n++] != 0x10) break;
+	       if (session->inbuffer[n++] != 0x03) break;
+	       crc &= 0xff;
+
+	       if (crc != checksum) {
+		  gpsd_report(4, "EverMore checksum failed: %02x != %02x\n", crc, checksum);
+		  break;
+	       }
+	       ok = true;
+	    } while (0);
+	    /*@ +charint */
+
+	    if (ok)
+		packet_accept(session, EVERMORE_PACKET);
+	    else
+		session->packet_state = GROUND_STATE;
+	    packet_discard(session);
+            break;
+#endif /* EVERMORE_ENABLE */
+#ifdef ITALK_ENABLE
+	} else if (session->packet_state == ITALK_RECOGNIZED) {
+	    u_int16_t len, n, sum;
+	    len = (unsigned short)(session->packet_length / 2 - 1);
+	    /*
+	     * Skip first 9 words so we compute checksum only over data
+	     * portion of packet.
+	     */
+	    for (n = sum = 0; n < (unsigned short)(len - 9); n++)
+		sum += getword(9 + n);
+	    if (len == 0 || sum == (u_int16_t)getword(len+1)) {
+		packet_accept(session, ITALK_PACKET);
+	    } else
+		session->packet_state = GROUND_STATE;
+	    packet_discard(session);
+            break;
+#endif /* ITALK_ENABLE */
+#ifdef RTCM104_ENABLE
+	} else if (session->packet_state == RTCM_RECOGNIZED) {
+	    /*
+	     * RTCM packets don't have checksums.  The six bits of parity 
+	     * per word and the preamble better be good enough.
+	     */
+	    packet_accept(session, RTCM_PACKET);
+	    session->packet_state = RTCM_SYNC_STATE;
+	    packet_discard(session);
+            break;
+#endif /* RTCM104_ENABLE */
+	}
+    } /* while */
+
+    return (ssize_t)newdata;
+}
+#undef getword
+
+ssize_t packet_get(struct gps_device_t *session)
+/* grab a packet; returns ether BAD_PACKET or the length */
+{
+    ssize_t newdata;
+    /*@ -modobserver @*/
+    newdata = read(session->gpsdata.gps_fd, session->inbuffer+session->inbuflen,
+			sizeof(session->inbuffer)-(session->inbuflen));
+    /*@ +modobserver @*/
+    if (newdata < 0 && errno != EAGAIN)
+	return BAD_PACKET;
+    else if (newdata == 0 || (newdata < 0 && errno == EAGAIN))
+	return 0;
+    return packet_parse(session, (size_t)newdata);
 }
 
 void packet_reset(struct gps_device_t *session)
@@ -610,8 +808,10 @@ void packet_reset(struct gps_device_t *session)
     session->packet_state = GROUND_STATE;
     session->inbuflen = 0;
     session->inbufptr = session->inbuffer;
+    isgps_init(session);
 }
 
+
 #ifdef __UNUSED__
 void packet_pushback(struct gps_device_t *session)
 /* push back the last packet grabbed */
@@ -628,177 +828,3 @@ void packet_pushback(struct gps_device_t *session)
     }
 }
 #endif /* __UNUSED */
-
-#ifdef TESTMAIN
-/* To build a test main, compile with cc -DTESTMAIN -g packet.c -o packet */
-
-int main(int argc, char *argv[])
-{
-    struct map {
-	char		*legend;
-	unsigned char	test[MAX_PACKET_LENGTH+1];
-	int		testlen;
-	int		garbage_offset;
-	int		type;
-	int		initstate;
-    };
-    struct map tests[] = {
-	/* NMEA tests */
-	{
-	    "NMEA packet with checksum (1)",
-	    "$GPVTG,308.74,T,,M,0.00,N,0.0,K*68\r\n",
-	    36,
-	    0,
-	    NMEA_PACKET,
-	    GROUND_STATE,
-	},
-	{
-	    "NMEA packet with checksum (2)",
-	    "$GPGGA,110534.994,4002.1425,N,07531.2585,W,0,00,50.0,172.7,M,-33.8,M,0.0,0000*7A\r\n",
-	    82,
-	    0,
-	    NMEA_PACKET,
-	    GROUND_STATE,
-	},
-	{
-	    "NMEA packet with checksum and 4 chars of leading garbage",
-	    "\xff\xbf\x00\xbf$GPVTG,308.74,T,,M,0.00,N,0.0,K*68\r\n",
-	    40,
-	    4,
-	    NMEA_PACKET,
-	    GROUND_STATE,
-	},
-	{
-	    "NMEA packet without checksum",
-	    "$PSRF105,1\r\n",
-	    12,
-	    0,
-	    NMEA_PACKET,
-	    GROUND_STATE,
-	},
-	{
-	    "NMEA packet with wrong checksum",
-	    "$GPVTG,308.74,T,,M,0.00,N,0.0,K*28\r\n",
-	    36,
-	    0,
-	    BAD_PACKET,
-	    GROUND_STATE,
-	},
-	/* SiRF tests */
-	{
-	    "SiRF WAAS version ID",
-	    {
-		0xA0, 0xA2, 0x00, 0x15, 
-		0x06, 0x06, 0x31, 0x2E, 0x32, 0x2E, 0x30, 0x44, 
-		0x4B, 0x49, 0x54, 0x31, 0x31, 0x39, 0x20, 0x53, 
-		0x4D, 0x00, 0x00, 0x00, 0x00,
-		0x03, 0x82, 0xB0, 0xB3},
-	    29,
-	    0,
-	    SIRF_PACKET,
-	    GROUND_STATE,
-	},
-	{
-	    "SiRF WAAS version ID with 3 chars of leading garbage",
-	    {
-		0xff, 0x00, 0xff,
-		0xA0, 0xA2, 0x00, 0x15, 
-		0x06, 0x06, 0x31, 0x2E, 0x32, 0x2E, 0x30, 0x44, 
-		0x4B, 0x49, 0x54, 0x31, 0x31, 0x39, 0x20, 0x53, 
-		0x4D, 0x00, 0x00, 0x00, 0x00,
-		0x03, 0x82, 0xB0, 0xB3},
-	    32,
-	    3,
-	    SIRF_PACKET,
-	    GROUND_STATE,
-	},
-	{
-	    "SiRF WAAS version ID with wrong checksum",
-	    {
-		0xA0, 0xA2, 0x00, 0x15, 
-		0x06, 0x06, 0x31, 0x2E, 0x32, 0x2E, 0x30, 0x44, 
-		0x4B, 0x49, 0x54, 0x31, 0x31, 0x39, 0x20, 0x53, 
-		0x4D, 0x00, 0x00, 0x00, 0x00,
-		0x03, 0x00, 0xB0, 0xB3},
-	    29,
-	    0,
-	    BAD_PACKET,
-	    GROUND_STATE,
-	},
-	{
-	    "SiRF WAAS version ID with bad length",
-	    {
-		0xA0, 0xA2, 0xff, 0x15, 
-		0x06, 0x06, 0x31, 0x2E, 0x32, 0x2E, 0x30, 0x44, 
-		0x4B, 0x49, 0x54, 0x31, 0x31, 0x39, 0x20, 0x53, 
-		0x4D, 0x00, 0x00, 0x00, 0x00,
-		0x03, 0x82, 0xB0, 0xB3},
-	    29,
-	    0,
-	    BAD_PACKET,
-	    GROUND_STATE,
-	},
-	/* Zodiac tests */
-	{
-	    "Zodiac binary 1000 Geodetic Status Output Message",
-	    {
-		0xff, 0x81, 0xe8, 0x03, 0x31, 0x00, 0x00, 0x00, 0xe8, 0x79, 
-		0x74, 0x0e, 0x00, 0x00, 0x24, 0x00, 0x24, 0x00, 0x04, 0x00, 
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x90, 0x03, 0x23, 0x00, 
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1d, 0x00, 0x06, 0x00, 
-		0xcd, 0x07, 0x00, 0x00, 0x00, 0x00, 0x16, 0x00, 0x7b, 0x0d, 
-		0x00, 0x00, 0x12, 0x6b, 0xa7, 0x04, 0x41, 0x75, 0x32, 0xf8, 
-		0x03, 0x1f, 0x00, 0x00, 0xe6, 0xf2, 0x00, 0x00, 0x00, 0x00, 
-		0x00, 0x00, 0x11, 0xf6, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x40, 
-		0xd9, 0x12, 0x90, 0xd0, 0x03, 0x00, 0x00, 0xa3, 0xe1, 0x11, 
-		0x10, 0x27, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa3, 0xe1, 0x11, 
-		0x00, 0x00, 0x00, 0x00, 0xe0, 0x93, 0x04, 0x00, 0x04, 0xaa},
-	    110,
-	    0,
-	    ZODIAC_PACKET,
-	    ZODIAC_EXPECTED,
-	},
-    };
-
-    struct map *mp;
-    struct gps_device_t state;
-    int st;
-    unsigned char *cp;
-
-    if (argc > 1)
-	verbose = atoi(argv[1]); 
-
-    for (mp = tests; mp < tests + sizeof(tests)/sizeof(tests[0]); mp++) {
-	state.packet_type = BAD_PACKET;
-	state.packet_state = mp->initstate;
-	state.inbuflen = 0;
-	memcpy(state.inbufptr = state.inbuffer, mp->test, mp->testlen);
-	gpsd_report(2, "%s starts with state %d\n", mp->legend, mp->initstate);
-	waiting = mp->testlen;
-	st = packet_get(&state);
-	if (state.packet_type != mp->type)
-	    printf("%s test FAILED (packet type %d wrong).\n", mp->legend, st);
-	else if (memcmp(mp->test + mp->garbage_offset, state.outbuffer, state.outbuflen))
-	    printf("%s test FAILED (data garbled).\n", mp->legend);
-	else
-	    printf("%s test succeeded.\n", mp->legend);
-#ifdef DUMPIT
-	for (cp = state.outbuffer; 
-	     cp < state.outbuffer + state.outbuflen; 
-	     cp++) {
-	    if (st != NMEA_PACKET)
-		(void)printf(" 0x%02x", *cp);
-	    else if (*cp == '\r')
-		(void)fputs("\\r", stdout);
-	    else if (*cp == '\n')
-		(void)fputs("\\n", stdout);
-	    else if (isprint(*cp))
-		(void)putchar(*cp);
-	    else
-		(void)printf("\\x%02x", *cp);
-	}
-	(void)putchar('\n');
-#endif /* DUMPIT */
-    }
-}
-#endif /* TESTMAIN */
