@@ -18,6 +18,10 @@
  * accelerometer inputs to the chip to allow it to continue 
  * to navigate in the absence of satellite information, and 
  * to improve fixes when you do have satellites.)"
+ *
+ * "[When we need RINEX data, we can get it from] SiRF Message #5.
+ *  If it's no longer implemented on your receiver, messages
+ * 7, 28, 29 and 30 will give you the same information."
  */
 
 #include <stdio.h>
@@ -37,7 +41,7 @@
 #define HI(n)		((n) >> 8)
 #define LO(n)		((n) & 0xff)
 
-static bool sirf_write(int fd, unsigned char *msg) {
+bool sirf_write(int fd, unsigned char *msg) {
    unsigned int       crc;
    size_t    i, len;
    char	     buf[MAX_PACKET_LENGTH*2];
@@ -147,15 +151,10 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
     if (len == 0)
 	return 0;
 
-    buf2[0] = '\0';
-    for (i = 0; i < (int)len; i++)
-	(void)snprintf(buf2+strlen(buf2), 
-		       sizeof(buf2)-strlen(buf2),
-		       "%02x", (unsigned int)buf[i]);
-    strcat(buf2, "\n");
     buf += 4;
     len -= 8;
-    gpsd_report(5, "Raw SiRF packet type 0x%02x length %d: %s\n", buf[0],len,buf2);
+    gpsd_report(5, "Raw SiRF packet type 0x%02x length %d: %s\n", buf[0],len,
+		gpsd_hexdump(buf, len));
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag),
 		   "MID%d",(int)buf[0]);
 
@@ -167,12 +166,11 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	memset(session->gpsdata.used,0,sizeof(session->gpsdata.used));
 	for (i = 0; i < SIRF_CHANNELS; i++)
 	    session->gpsdata.used[i] = (int)getub(buf, 29+i);
-	if ((session->sirf.driverstate & (SIRF_GE_232 | UBLOX))==0) {
+	if ((session->driver.sirf.driverstate & (SIRF_GE_232 | UBLOX))==0) {
 	    /* position/velocity is bytes 1-18 */
 	    ecef_to_wgs84fix(&session->gpsdata, 
 			     getsl(buf, 1)*1.0, getsl(buf, 5)*1.0, getsl(buf, 9)*1.0,
 			     getsw(buf, 13)/8.0, getsw(buf, 15)/8.0, getsw(buf, 17)/8.0);
-	    /* WGS 84 geodesy parameters */
 	    /* fix status is byte 19 */
 	    navtype = (unsigned short)getub(buf, 19);
 	    session->gpsdata.status = STATUS_NO_FIX;
@@ -195,17 +193,17 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 		= gpstime_to_unix(getsw(buf, 22), getul(buf, 24)*1e-2) - session->context->leap_seconds;
 #ifdef NTPSHM_ENABLE
 	    if (session->gpsdata.newdata.mode > MODE_NO_FIX) {
-		if ((session->sirf.time_seen & TIME_SEEN_GPS_2) == 0)
+		if ((session->driver.sirf.time_seen & TIME_SEEN_GPS_2) == 0)
 		    gpsd_report(4, "valid time in message 0x02, seen=0x%02x\n",
-				session->sirf.time_seen);
-		session->sirf.time_seen |= TIME_SEEN_GPS_2;
-		if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_GPS_2))
+				session->driver.sirf.time_seen);
+		session->driver.sirf.time_seen |= TIME_SEEN_GPS_2;
+		if (IS_HIGHEST_BIT(session->driver.sirf.time_seen,TIME_SEEN_GPS_2))
 		    (void)ntpshm_put(session, session->gpsdata.newdata.time + 0.8);
 	    }
 #endif /* NTPSHM_ENABLE */
 	    /* fix quality data */
 	    session->gpsdata.hdop = (double)getub(buf, 20)/5.0;
-	    mask |= TIME_SET | LATLON_SET | TRACK_SET | SPEED_SET | STATUS_SET | MODE_SET | HDOP_SET | CYCLE_START_SET;
+	    mask |= TIME_SET | LATLON_SET | TRACK_SET | SPEED_SET | STATUS_SET | MODE_SET | HDOP_SET | USED_SET | CYCLE_START_SET;
 	}
 	return mask;
 
@@ -241,11 +239,11 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	session->gpsdata.satellites = st;
 #ifdef NTPSHM_ENABLE
 	if (st > 3) {
-	    if ((session->sirf.time_seen & TIME_SEEN_GPS_1)==0)
+	    if ((session->driver.sirf.time_seen & TIME_SEEN_GPS_1)==0)
 		gpsd_report(4, "valid time in message 0x04, seen=0x%02x\n",
-			    session->sirf.time_seen);
-	    session->sirf.time_seen |= TIME_SEEN_GPS_1;
-	    if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_GPS_1))
+			    session->driver.sirf.time_seen);
+	    session->driver.sirf.time_seen |= TIME_SEEN_GPS_1;
+	    if (IS_HIGHEST_BIT(session->driver.sirf.time_seen,TIME_SEEN_GPS_1))
 		(void)ntpshm_put(session,session->gpsdata.sentence_time+0.8);
 	}
 #endif /* NTPSHM_ENABLE */
@@ -254,7 +252,7 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	 * rate-control command for 04, at least at firmware rev. 231, 
 	 * so we have to do our own rate-limiting here...
 	 */
-	if ((session->sirf.satcounter++ % 5) != 0)
+	if ((session->driver.sirf.satcounter++ % 5) != 0)
 	    break;
 	gpsd_report(4, "MTD 0x04: %d satellites\n", st);
 	return TIME_SET | SATELLITE_SET;
@@ -266,11 +264,11 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	gpsd_report(4, "FV  0x06: Firmware version: %s\n", buf+1);
 	fv = atof((char *)(buf+1));
 	if (fv < 231) {
-	    session->sirf.driverstate |= SIRF_LT_231;
+	    session->driver.sirf.driverstate |= SIRF_LT_231;
 	    if (fv > 200)
 		sirfbin_mode(session, 0);
 	} else if (fv < 232) 
-	    session->sirf.driverstate |= SIRF_EQ_231;
+	    session->driver.sirf.driverstate |= SIRF_EQ_231;
 	else {
 	    /*@ +charint @*/
 	    unsigned char enablemid52[] = {
@@ -280,28 +278,21 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	    /*@ -charint @*/
 	    gpsd_report(4, "Enabling PPS message...\n");
 	    (void)sirf_write(session->gpsdata.gps_fd, enablemid52);
-	    session->sirf.driverstate |= SIRF_GE_232;
+	    session->driver.sirf.driverstate |= SIRF_GE_232;
 	    session->context->valid |= LEAP_SECOND_VALID;
 	}
 	if (strstr((char *)(buf+1), "ES"))
 	    gpsd_report(4, "Firmware has XTrac capability\n");
-	gpsd_report(4, "Driver state flags are: %0x\n", session->sirf.driverstate);
-	session->sirf.time_seen = 0;
+	gpsd_report(4, "Driver state flags are: %0x\n", session->driver.sirf.driverstate);
+	session->driver.sirf.time_seen = 0;
 	if ((session->context->valid & LEAP_SECOND_VALID)==0) {
 	    gpsd_report(4, "Enabling subframe transmission...\n");
 	    (void)sirf_write(session->gpsdata.gps_fd, enablesubframe);
 	}
 	return 0;
 
-    case 0x08:
+    case 0x08:		/* subframe data -- extract leap-second from this */
 	/*
-	 * Heavy black magic begins here!
-	 *
-	 * A description of how to decode these bits is at
-	 * <http://home-2.worldonline.nl/~samsvl/nav2eu.htm>
-	 *
-	 * We're after subframe 4 page 18 word 9, the leap year correction.
-	 *
 	 * Chris Kuethe says:
 	 * "Message 8 is generated as the data is received. It is not
 	 * buffered on the chip. So when you enable message 8, you'll
@@ -311,10 +302,10 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	 * stored, which is really lame, as the UTC-GPS correction
 	 * changes 1 second every few years. Maybe."
 	 */
-        {
-	    unsigned int pageid, subframe, leap, words[10];
-	    unsigned int chan = (unsigned int)getub(buf, 1);
-	    unsigned int svid = (unsigned int)getub(buf, 2);
+       {
+	    unsigned int words[10];
+	    //unsigned int chan = (unsigned int)getub(buf, 1);
+	    //unsigned int svid = (unsigned int)getub(buf, 2);
 	    words[0] = (unsigned int)getul(buf, 3);
 	    words[1] = (unsigned int)getul(buf, 7);
 	    words[2] = (unsigned int)getul(buf, 11);
@@ -325,85 +316,7 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	    words[7] = (unsigned int)getul(buf, 31);
 	    words[8] = (unsigned int)getul(buf, 35);
 	    words[9] = (unsigned int)getul(buf, 39);
-	    gpsd_report(4, "50B (raw): CH=%d, SV=%d %08x %08x %08x %08x %08x %08x %08x %08x %08x %08x\n", 
-	    	chan, svid, 
-	    	words[0], words[1], words[2], words[3], words[4], 
-	    	words[5], words[6], words[7], words[8], words[9]);
-	    /*
-	     * Mask off the high 2 bits and shift out the 6 parity bits.
-	     * Once we've filtered, we can ignore the TEL and HOW words.
-	     * We don't need to check parity here, the SiRF chipset does
-	     * that and throws a subframe error if the parity is wrong.
-	     */
-	    for (i = 0; i < 10; i++)
-		words[i] = (words[i]  & 0x3fffffff) >> 6;
-	    /*
-	     * "First, throw away everything that doesn't start with 8b or
-	     * 74. more correctly the first byte should be 10001011. If
-	     * it's 01110100, then you have a subframe with inverted
-	     * polarity and each byte needs to be xored against 0xff to
-	     * remove the inversion."
-	     */
-	    words[0] &= 0xff0000;
-	    if (words[0] != 0x8b0000 && words[0] != 0x740000)
-	    	break;
-	    if (words[0] == 0x740000)
-		for (i = 1; i < 10; i++)
-		    words[i] ^= 0xffffff;
-	    /*
-	     * The subframe ID is in the Hand Over Word (page 80) 
-	     */
-	    subframe = ((words[1] >> 2) & 0x07);
-	    /* we're not interested in anything but subframe 4 */
-	    if (subframe != 4)
-		break;
-	    /*
-	     * Pages 66-76a,80 of ICD-GPS-200 are the subframe structures.
-	     * Subframe 4 page 18 is on page 74.
-	     * See page 105 for the mapping between magic SVIDs and pages.
-	     */
-	    pageid = (words[2] & 0x3F0000) >> 16;
-	    gpsd_report(2, "Subframe 4 SVID is %d\n", pageid);
-	    if (pageid == 56) {	/* magic SVID for page 18 */
-		/* once we've filtered, we can ignore the TEL and HOW words */
-		gpsd_report(2, "50B: CH=%d, SV=%d SF=%d %06x %06x %06x %06x %06x %06x %06x %06x\n", 
-			    chan, svid, subframe,
-				words[2], words[3], words[4], words[5], 
-				words[6], words[7], words[8], words[9]);
-		leap = (words[8] & 0xff0000) >> 16;
-		/*
-		 * There appears to be some bizarre bug that randomly causes
-		 * this field to come out two's-complemented.  Work around
-		 * this.  At the current expected rate of issuing leap-seconds
-		 * this kluge won't bite until about 2070, by which time SiRF
-		 * had better have fixed their damn firmware...
-		 *
-		 * Carl: ...I am unsure, and suggest you
-		 * experiment.  The D30 bit is in bit 30 of the 32-bit
-		 * word (next to MSB), and should signal an inverted
-		 * value when it is one coming over the air.  But if
-		 * the bit is set and the word decodes right without
-		 * inversion, then we properly caught it.  Cases where
-		 * you see subframe 6 rather than 1 means we should
-		 * have done the inversion but we did not.  Some other
-		 * things you can watch for: in any subframe, the
-		 * second word (HOW word) should have last 2 parity
-		 * bits 00 -- there are bits within the rest of the
-		 * word that are set as required to ensure that.  The
-		 * same goes for word 10.  That means that both words
-		 * 1 and 3 (the words that immediately follow words 10
-		 * and 2, respectively) should always be uninverted.
-		 * In these cases, the D29 and D30 from the previous
-		 * words, found in the two MSBs of the word, should
-		 * show 00 -- if they don't then you may find an
-		 * unintended inversion due to noise on the data link.
-		 */
-		if (leap > 128)
-		    leap ^= 0xff;
-		gpsd_report(2, "leap-seconds is %d\n", leap);
-		session->context->leap_seconds = (int)leap;
-		session->context->valid |= LEAP_SECOND_VALID;
-	    }
+	    gpsd_interpret_subframe(session, words);
 
 	    if (session->context->valid & LEAP_SECOND_VALID) {
 		gpsd_report(4, "Disabling subframe transmission...\n");
@@ -451,11 +364,53 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	return 0;
 
     case 0x1b:		/* DGPS status (undocumented) */
+	/******************************************************************
+	 Not actually documented in any published materials.
+	 Here is what Chris Kuethe got from the SiRF folks:
+
+	Start of message
+	----------------
+	Message ID          1 byte    27
+	Correction Source   1 byte    0=None, 1=SBAS, 2=Serial, 3=Beacon,
+	4=Software
+
+	total:              2 bytes
+
+	Middle part of message varies if using beacon or other:
+	-------------------------------------------------------
+	If Beacon:
+	Receiver Freq Hz    4 bytes
+	Bit rate BPS        1 byte
+	Status bit map      1 byte    01=Signal Valid,
+				     02=Auto frequency detect
+				     04=Auto bit rate detect
+	Signal Magnitude    4 bytes   Note: in internal units
+	Signal Strength dB  2 bytes   derived from Signal Magnitude
+	SNR  dB             2 bytes
+
+	total:             14 bytes
+
+	If Not Beacon:
+	Correction Age[12]  1 byte x 12  Age in seconds in same order as follows
+	Reserved            2 bytes
+
+	total:             14 bytes
+
+	End of Message
+	--------------
+	Repeated 12 times (pad with 0 if less than 12 SV corrections):
+	SVID                1 byte
+	Correction (m)      1 byte
+
+	total               2 x 12 = 24 bytes
+	******************************************************************/
 	return 0;
 
     case 0x29:		/* Geodetic Navigation Information */
 	mask = 0;
-	if (session->sirf.driverstate & SIRF_GE_232) {
+	if (session->driver.sirf.driverstate & SIRF_GE_232) {
+	    struct tm unpacked_date;
+	    double subseconds;
 	    /*
 	     * Many versions of the SiRF protocol manual don't document 
 	     * this sentence at all.  Those that do may incorrectly
@@ -500,23 +455,25 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	     * UTC second     2               2
 	     *                11              8
 	     */
-	    session->gpsdata.nmea_date.tm_year = (int)getuw(buf, 11);
-	    session->gpsdata.nmea_date.tm_mon = (int)getub(buf, 13)-1;
-	    session->gpsdata.nmea_date.tm_mday = (int)getub(buf, 14);
-	    session->gpsdata.nmea_date.tm_hour = (int)getub(buf, 15);
-	    session->gpsdata.nmea_date.tm_min = (int)getub(buf, 16);
-	    session->gpsdata.nmea_date.tm_sec = 0;
-	    session->gpsdata.subseconds = getuw(buf, 17)*1e-3;
+	    unpacked_date.tm_year = (int)getuw(buf, 11);
+	    unpacked_date.tm_mon = (int)getub(buf, 13)-1;
+	    unpacked_date.tm_mday = (int)getub(buf, 14);
+	    unpacked_date.tm_hour = (int)getub(buf, 15);
+	    unpacked_date.tm_min = (int)getub(buf, 16);
+	    unpacked_date.tm_sec = 0;
+	    subseconds = getuw(buf, 17)*1e-3;
+	    /*@ -compdef */
 	    session->gpsdata.newdata.time = session->gpsdata.sentence_time
-		= (double)mktime(&session->gpsdata.nmea_date)+session->gpsdata.subseconds;
+		= (double)mktime(&unpacked_date)+subseconds;
+	    /*@ +compdef */
 	    gpsd_report(5, "MID 41 UTC: %lf\n", session->gpsdata.newdata.time);
 #ifdef NTPSHM_ENABLE
-	    if (session->gpsdata.newdata.mode > MODE_NO_FIX && session->gpsdata.nmea_date.tm_year != 0) {
-		if ((session->sirf.time_seen & TIME_SEEN_UTC_1) == 0)
+	    if (session->gpsdata.newdata.mode > MODE_NO_FIX && unpacked_date.tm_year != 0) {
+		if ((session->driver.sirf.time_seen & TIME_SEEN_UTC_1) == 0)
 		    gpsd_report(4, "valid time in message 0x29, seen=0x%02x\n",
-				session->sirf.time_seen);
-		session->sirf.time_seen |= TIME_SEEN_UTC_1;
-		if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_UTC_1))
+				session->driver.sirf.time_seen);
+		session->driver.sirf.time_seen |= TIME_SEEN_UTC_1;
+		if (IS_HIGHEST_BIT(session->driver.sirf.time_seen,TIME_SEEN_UTC_1))
 		    (void)ntpshm_put(session, session->gpsdata.newdata.time + 0.8);
 	    }
 #endif /* NTPSHM_ENABLE */
@@ -566,20 +523,25 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	mask = 0;
 	gpsd_report(4, "PPS 0x34: Status = 0x%02x\n", getub(buf, 14));
 	if (((int)getub(buf, 14) & 0x07) == 0x07) {	/* valid UTC time? */
-	    session->gpsdata.nmea_date.tm_hour = (int)getub(buf, 1);
-	    session->gpsdata.nmea_date.tm_min = (int)getub(buf, 2);
-	    session->gpsdata.nmea_date.tm_sec = (int)getub(buf, 3);
-	    session->gpsdata.nmea_date.tm_mday = (int)getub(buf, 4);
-	    session->gpsdata.nmea_date.tm_mon = (int)getub(buf, 5) - 1;
-	    session->gpsdata.nmea_date.tm_year = (int)getuw(buf, 6) - 1900;
+	    struct tm unpacked_date;
+	    unpacked_date.tm_hour = (int)getub(buf, 1);
+	    unpacked_date.tm_min = (int)getub(buf, 2);
+	    unpacked_date.tm_sec = (int)getub(buf, 3);
+	    unpacked_date.tm_mday = (int)getub(buf, 4);
+	    unpacked_date.tm_mon = (int)getub(buf, 5) - 1;
+	    unpacked_date.tm_year = (int)getuw(buf, 6) - 1900;
+	    /*@ -compdef */
+	    session->gpsdata.newdata.time = session->gpsdata.sentence_time
+		= (double)mktime(&unpacked_date);
+	    /*@ +compdef */
 	    session->context->leap_seconds = (int)getuw(buf, 8);
 	    session->context->valid |= LEAP_SECOND_VALID;
 #ifdef NTPSHM_ENABLE
-	    if ((session->sirf.time_seen & TIME_SEEN_UTC_2) == 0)
+	    if ((session->driver.sirf.time_seen & TIME_SEEN_UTC_2) == 0)
 		gpsd_report(4, "valid time in message 0x34, seen=0x%02x\n",
-				session->sirf.time_seen);
-	    session->sirf.time_seen |= TIME_SEEN_UTC_2;
-	    if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_UTC_2))
+				session->driver.sirf.time_seen);
+	    session->driver.sirf.time_seen |= TIME_SEEN_UTC_2;
+	    if (IS_HIGHEST_BIT(session->driver.sirf.time_seen,TIME_SEEN_UTC_2))
 		(void)ntpshm_put(session, session->gpsdata.newdata.time + 0.3);
 #endif /* NTPSHM_ENABLE */
 	    mask |= TIME_SET;
@@ -613,22 +575,26 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 		    navtype, session->gpsdata.status, session->gpsdata.newdata.mode);
 
 	if (navtype & 0x40) {		/* UTC corrected timestamp? */
+	    struct tm unpacked_date;
+	    double subseconds;
 	    mask |= TIME_SET;
-	    session->gpsdata.nmea_date.tm_year = (int)getuw(buf, 26) - 1900;
-	    session->gpsdata.nmea_date.tm_mon = (int)getub(buf, 28) - 1;
-	    session->gpsdata.nmea_date.tm_mday = (int)getub(buf, 29);
-	    session->gpsdata.nmea_date.tm_hour = (int)getub(buf, 30);
-	    session->gpsdata.nmea_date.tm_min = (int)getub(buf, 31);
-	    session->gpsdata.nmea_date.tm_sec = 0;
-	    session->gpsdata.subseconds = ((unsigned short)getuw(buf, 32))*1e-3;
+	    unpacked_date.tm_year = (int)getuw(buf, 26) - 1900;
+	    unpacked_date.tm_mon = (int)getub(buf, 28) - 1;
+	    unpacked_date.tm_mday = (int)getub(buf, 29);
+	    unpacked_date.tm_hour = (int)getub(buf, 30);
+	    unpacked_date.tm_min = (int)getub(buf, 31);
+	    unpacked_date.tm_sec = 0;
+	    subseconds = ((unsigned short)getuw(buf, 32))*1e-3;
+	    /*@ -compdef */
 	    session->gpsdata.newdata.time = session->gpsdata.sentence_time
-		= (double)mkgmtime(&session->gpsdata.nmea_date)+session->gpsdata.subseconds;
+		= (double)mkgmtime(&unpacked_date)+subseconds;
+	    /*@ +compdef */
 #ifdef NTPSHM_ENABLE
-	    if ((session->sirf.time_seen & TIME_SEEN_UTC_2) == 0)
+	    if ((session->driver.sirf.time_seen & TIME_SEEN_UTC_2) == 0)
 		gpsd_report(4, "valid time in message 0x62, seen=0x%02x\n",
-				session->sirf.time_seen);
-	    session->sirf.time_seen |= TIME_SEEN_UTC_2;
-	    if (IS_HIGHEST_BIT(session->sirf.time_seen,TIME_SEEN_UTC_2))
+				session->driver.sirf.time_seen);
+	    session->driver.sirf.time_seen |= TIME_SEEN_UTC_2;
+	    if (IS_HIGHEST_BIT(session->driver.sirf.time_seen,TIME_SEEN_UTC_2))
 		(void)ntpshm_put(session, session->gpsdata.newdata.time + 0.8);
 #endif /* NTPSHM_ENABLE */
 	    session->context->valid |= LEAP_SECOND_VALID;
@@ -639,7 +605,7 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	session->gpsdata.hdop = (int)getub(buf, 36) / 5.0;
 	session->gpsdata.vdop = (int)getub(buf, 37) / 5.0;
 	session->gpsdata.tdop = (int)getub(buf, 38) / 5.0;
-	session->sirf.driverstate |= UBLOX;
+	session->driver.sirf.driverstate |= UBLOX;
 	return mask;
 
     case 0xff:		/* Debug messages */
@@ -657,12 +623,8 @@ gps_mask_t sirf_parse(struct gps_device_t *session, unsigned char *buf, size_t l
 	return 0;
 
     default:
-	buf2[0] = '\0';
-	for (i = 0; i < (int)len; i++)
-	    (void)snprintf(buf2+strlen(buf2), 
-			   sizeof(buf2)-strlen(buf2),
-			   "%02x", (unsigned int)buf[i]);
-	gpsd_report(3, "Unknown SiRF packet id %d length %d: %s\n", buf[0], len, buf2);
+	gpsd_report(3, "Unknown SiRF packet id %d length %d: %s\n", 
+		    buf[0], len, gpsd_hexdump(buf, len));
 	return 0;
     }
 
@@ -677,10 +639,12 @@ static gps_mask_t sirfbin_parse_input(struct gps_device_t *session)
 	st = sirf_parse(session, session->outbuffer, session->outbuflen);
 	session->gpsdata.driver_mode = 1;
 	return st;
+#ifdef NMEA_ENABLE
     } else if (session->packet_type == NMEA_PACKET) {
-	st = nmea_parse((char *)session->outbuffer, &session->gpsdata);
+	st = nmea_parse((char *)session->outbuffer, session);
 	session->gpsdata.driver_mode = 0;
 	return st;
+#endif /* NMEA_ENABLE */
     } else
 	return 0;
 }
@@ -720,7 +684,19 @@ static void sirfbin_initializer(struct gps_device_t *session)
 				  0x00,		/* disable dead reckoning */
 				  0x01,		/* enable track smoothing */
 				 0x00, 0x00, 0xb0, 0xb3};
+	unsigned char requestecef[] = {0xa0, 0xa2, 0x00, 0x08,
+				       0xa6, 0x00, 0x02, 0x01,
+				       0x00, 0x00, 0x00, 0x00,
+				       0x00, 0x00, 0xb0, 0xb3};
+	unsigned char requesttracker[] = {0xa0, 0xa2, 0x00, 0x08, 
+					  0xa6, 0x00, 0x04, 0x03,
+					  0x00, 0x00, 0x00, 0x00,
+					  0x00, 0x00, 0xb0, 0xb3};
 	/*@ -charint @*/
+	gpsd_report(4, "Requesting periodic ecef reports...\n");
+	(void)sirf_write(session->gpsdata.gps_fd, requestecef);
+	gpsd_report(4, "Requesting periodic tracker reports...\n");
+	(void)sirf_write(session->gpsdata.gps_fd, requesttracker);
 	gpsd_report(4, "Setting DGPS control to use SBAS...\n");
 	(void)sirf_write(session->gpsdata.gps_fd, dgpscontrol);
 	gpsd_report(4, "Setting SBAS to auto/integrity mode...\n");
@@ -740,16 +716,19 @@ static bool sirfbin_speed(struct gps_device_t *session, speed_t speed)
 /* this is everything we export */
 struct gps_type_t sirf_binary =
 {
-    "SiRF-II binary",		/* full name of type */
-    "$Ack Input105.",		/* expected response to SiRF PSRF105 */
-    NULL,			/* no probe */
-    sirfbin_initializer,	/* initialize the device */
-    packet_get,			/* how to grab a packet */
-    sirfbin_parse_input,	/* read and parse message packets */
-    NULL,			/* send DGPS correction */
-    sirfbin_speed,		/* we can change baud rate */
-    sirfbin_mode,		/* there's a mode switcher */
-    NULL,			/* caller needs to supply a close hook */
-    1,				/* updates every second */
+    .typename       = "SiRF-II binary",	/* full name of type */
+    .trigger        = "$Ack Input105.",	/* expected response to SiRF PSRF105 */
+    .channels       = SIRF_CHANNELS,	/* consumer-grade GPS */
+    .probe          = NULL,		/* no probe */
+    .initializer    = sirfbin_initializer,/* initialize the device */
+    .get_packet     = packet_get,	/* use the generic packet getter */
+    .parse_packet   = sirfbin_parse_input,/* parse message packets */
+    .rtcm_writer    = pass_rtcm,	/* send RTCM data straight */
+    .speed_switcher = sirfbin_speed,	/* we can change baud rate */
+    .mode_switcher  = sirfbin_mode,	/* there's a mode switcher */
+    .rate_switcher  = NULL,		/* no sample-rate switcher */
+    .cycle_chars    = -1,		/* not relevant, no rate switch */
+    .wrapup         = NULL,		/* no close hook */
+    .cycle          = 1,		/* updates every second */
 };
 #endif /* defined(SIRFII_ENABLE) && defined(BINARY_ENABLE) */
