@@ -38,7 +38,7 @@
 #include "config.h"
 #include "gps.h"	/* for DEFAULT_GPSD_PORT; brings in PI as well */
 
-#define PUT_ORIGIN	4
+#define PUT_ORIGIN	-4
 #include "bits.h"
 
 #if defined(HAVE_SYS_TIME_H)
@@ -142,10 +142,12 @@ static int nmea_send(int fd, const char *fmt, ... )
     va_end(ap);
     strcat(buf, "*");
     nmea_add_checksum(buf);
+    (void)fputs(buf, stderr);		/* so user can watch the baud hunt */
     status = (size_t)write(fd, buf, strlen(buf));
     if (status == strlen(buf)) {
 	return (int)status;
     } else {
+	perror("SiRF write");
 	return -1;
     }
 }
@@ -654,7 +656,7 @@ static int set_speed(unsigned int speed, unsigned int stopbits)
 	}
 	/*@ -charint @*/
     }
-
+    
     return NO_PACKET;
 }
 
@@ -676,8 +678,7 @@ static unsigned int hunt_open(unsigned int *pstopbits)
 
     for (trystopbits = 1; trystopbits <= 2; trystopbits++) {
 	*pstopbits = trystopbits;
-	for (ip = rates; ip < rates + sizeof(rates)/sizeof(rates[0]); ip++)
-	{
+	for (ip = rates; ip < rates + sizeof(rates)/sizeof(rates[0]); ip++) {
 	    if ((st = set_speed(*ip, trystopbits)) == SIRF_PACKET)
 		return get_speed(&ttyset);
 	    else if (st == NMEA_PACKET) {
@@ -690,20 +691,18 @@ static unsigned int hunt_open(unsigned int *pstopbits)
     return 0;
 }
 
-static int serial_initialize(char *device)
+static void serial_initialize(char *device)
 {
-    if ((devicefd = open(device,O_RDWR)) < 0) {
+    if ((controlfd = devicefd = open(device,O_RDWR)) < 0) {
 	perror(device);
 	exit(1);
     }
-    
+
     /* Save original terminal parameters */
     if (tcgetattr(devicefd, &ttyset) != 0 || (bps = hunt_open(&stopbits))==0) {
 	(void)fputs("Can't sync up with device!\n", stderr);
 	exit(1);
     }
-
-    return devicefd;
 }
 
 
@@ -921,16 +920,21 @@ int main (int argc, char **argv)
 
     gmt_offset = (int)tzoffset();
 
-    while ((option = getopt(argc, argv, "hvF:")) != -1) {
+    /*@ -branchstate @*/
+    while ((option = getopt(argc, argv, "F:vh")) != -1) {
 	switch (option) {
+	case 'F':
+	    controlsock = optarg;
+	    break;
 	case 'v':
 	    (void)printf("sirfmon %s\n", VERSION);
 	    exit(0);
 	case 'h': case '?': default:
-	    (void)fputs("usage:  sirfmon [-?hv] [server[:port:[device]]]\n", stderr);
+	    (void)fputs("usage:  sirfmon [-?hv] [-F controlsock] [server[:port:[device]]]\n", stderr);
 	    exit(1);
 	}
     }
+    /*@ +branchstate @*/
     /*@ -nullpass -branchstate @*/
     if (optind < argc) {
 	arg = strdup(argv[optind]);
@@ -978,7 +982,7 @@ int main (int argc, char **argv)
 	/*@ +compdef @*/
 	serial = false;
     } else {
-	devicefd = controlfd = serial_initialize(device = arg);
+	serial_initialize(device = arg);
 	serial = true;
     }
     /*@ +boolops */
