@@ -15,18 +15,13 @@
 #define SEMI_2_DEG	(180.0 / 2147483647)	/* 2^-31 semicircle to deg */
 
 #ifdef TSIP_ENABLE
+#define TSIP_CHANNELS	12
 
-static int tsip_write(int fd, unsigned int id, unsigned char *buf, int len)
+static int tsip_write(int fd, unsigned int id, unsigned char *buf, size_t len)
 {
-    int i;
     char buf2[BUFSIZ];
 
-    buf2[0] = '\0';
-    for (i = 0; i < len; i++)
-	(void)snprintf(buf2+strlen(buf2), 
-		       sizeof(buf2)-strlen(buf2), 
-		       "%02x", (unsigned)buf[i]);
-    gpsd_report(5, "Sent TSIP packet id 0x%02x: %s\n",id,buf2);
+    gpsd_report(5, "Sent TSIP packet id 0x%02x: %s\n",id,gpsd_hexdump(buf,len));
 
     /*@ +charint @*/
     buf2[0] = '\x10';
@@ -55,7 +50,9 @@ static void tsip_initializer(struct gps_device_t *session)
 {
     unsigned char buf[100];
 
-    /* TSIP is ODD parity 1 stopbit, change it */
+    /* TSIP is ODD parity 1 stopbit, save original values and change it */
+    session->driver.tsip.parity = session->gpsdata.parity;
+    session->driver.tsip.stopbits = session->gpsdata.stopbits;
     gpsd_set_speed(session, session->gpsdata.baudrate, 'O', 1);
 
     /* I/O Options */
@@ -77,6 +74,19 @@ static void tsip_initializer(struct gps_device_t *session)
     /* Request Current Datum Values */
     putbyte(buf,0,0x15);
     (void)tsip_write(session->gpsdata.gps_fd, 0x8e, buf, 1);
+
+    /* Request Navigation Configuration */
+    putbyte(buf,0,0x03);
+    (void)tsip_write(session->gpsdata.gps_fd, 0xbb, buf, 1);
+}
+
+static void tsip_wrapup(struct gps_device_t *session)
+{
+    /* restore saved parity and stopbits when leaving TSIP mode */
+    gpsd_set_speed(session, 
+		   session->gpsdata.baudrate,
+		   (unsigned char)session->driver.tsip.parity, 
+		   session->driver.tsip.stopbits);
 }
 
 static bool tsip_speed_switch(struct gps_device_t *session, unsigned int speed)
@@ -103,7 +113,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
     int i, j, len, count;
     gps_mask_t mask = 0;
     unsigned int id;
-    u_int8_t u1,u2,u3,u4;
+    u_int8_t u1,u2,u3,u4,u5;
     int16_t s1,s2,s3,s4;
     int32_t sl1,sl2,sl3;
     u_int32_t ul1,ul2;
@@ -164,12 +174,12 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
     case 0x41:		/* GPS Time */
 	if (len != 10)
 	    break;
-	session->tsip.last_41 = now;			/* keep timestamp for request */
+	session->driver.tsip.last_41 = now;			/* keep timestamp for request */
 	f1 = getf(buf,0);			/* gpstime */
 	s1 = getsw(buf,4);			/* week */
 	f2 = getf(buf,6);			/* leap seconds */
 	if (f1 >= 0.0 && f2 > 10.0) {
-	    session->tsip.gps_week = s1;
+	    session->driver.tsip.gps_week = s1;
 	    session->context->leap_seconds = (int)roundf(f2);
 	    session->context->valid |= LEAP_SECOND_VALID;
 
@@ -224,7 +234,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    u1 = getub(buf,5*i + 1);
 	    if ((f1 = getf(buf,5*i + 2)) < 0)
 		f1 = 0.0;
-	    for (j = 0; j < MAXCHANNELS; j++)
+	    for (j = 0; j < TSIP_CHANNELS; j++)
 		if (session->gpsdata.PRN[j] == (int)u1) {
 		    session->gpsdata.ss[j] = (int)roundf(f1);
 		    break;
@@ -247,9 +257,9 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	session->gpsdata.newdata.altitude  = getf(buf,8);
 	f1 = getf(buf,12);			/* clock bias */
 	f2 = getf(buf,16);			/* time-of-fix */
-	if (session->tsip.gps_week) {
+	if (session->driver.tsip.gps_week) {
 	    session->gpsdata.newdata.time = session->gpsdata.sentence_time =
-		gpstime_to_unix((int)session->tsip.gps_week, f2) - session->context->leap_seconds;
+		gpstime_to_unix((int)session->driver.tsip.gps_week, f2) - session->context->leap_seconds;
 	    mask |= TIME_SET;
 	}
 	gpsd_report(4, "GPS LLA %f %f %f\n",session->gpsdata.newdata.latitude,session->gpsdata.newdata.longitude,session->gpsdata.newdata.altitude);
@@ -263,7 +273,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	u3 = getub(buf,2);			/* Status 2 */
 	gpsd_report(4, "Machine ID %02x %02x %02x\n",u1,u2,u3);
 #if USE_SUPERPACKET
-	if ((u3 & 0x01) != (u_int8_t)0 && !session->tsip.superpkt) {
+	if ((u3 & 0x01) != (u_int8_t)0 && !session->driver.tsip.superpkt) {
 	    gpsd_report(2, "Switching to Super Packet mode\n");
 
 	    /* set new I/O Options for Super Packet output */
@@ -272,7 +282,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    putbyte(buf,2,0x00);		/* Time: GPS */
 	    putbyte(buf,3,0x08);		/* Aux: dBHz */
 	    (void)tsip_write(session->gpsdata.gps_fd, 0x35, buf, 4);
-	    session->tsip.superpkt = true;
+	    session->driver.tsip.superpkt = true;
 	}
 #endif /* USE_SUPERPACKET */
 	break;
@@ -324,7 +334,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	s1 = getsw(buf,6);			/* tsip.gps_week */
 	/*@ +charint @*/
 	if (getub(buf,0) == 0x01)		/* good current fix? */
-	    session->tsip.gps_week = s1;
+	    session->driver.tsip.gps_week = s1;
 	/*@ -charint @*/
 	gpsd_report(4, "Fix info %02x %02x %d %f\n",u1,u2,s1,f1);
 	break;
@@ -344,7 +354,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
     case 0x5c:		/* Satellite Tracking Status */
 	if (len != 24)
 	    break;
-	session->tsip.last_5c = now;		/* keep timestamp for request */
+	session->driver.tsip.last_5c = now;		/* keep timestamp for request */
 	u1 = getub(buf,0);			/* PRN */
 	u2 = getub(buf,1);			/* chan */
 	u3 = getub(buf,2);			/* Acquisition flag */
@@ -355,7 +365,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	d2 = getf(buf,16) * RAD_2_DEG;		/* Azimuth */
 	i = (int)(u2 >> 3);			/* channel number */
 	gpsd_report(4, "Satellite Tracking Status %d: %2d 0x%02x %d %d %.1f %f %.1f %.1f\n",i,u1,u2,u3,u4,f1,f2,d1,d2);
-	if (i < MAXCHANNELS) {
+	if (i < TSIP_CHANNELS) {
 	    if (d1 >= 0.0) {
 		session->gpsdata.PRN[i] = (int)u1;
 		session->gpsdata.ss[i] = (int)roundf(f1);
@@ -376,7 +386,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	count = (int)((u1 >> 4) & 0x0f);
 	if (len != (17 + count))
 	    break;
-	session->tsip.last_6d = now;		/* keep timestamp for request */
+	session->driver.tsip.last_6d = now;		/* keep timestamp for request */
 	switch (u1 & 7)				/* dimension */
 	{
 	case 3:
@@ -414,7 +424,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		session->gpsdata.hdop, session->gpsdata.vdop,
 		session->gpsdata.tdop, session->gpsdata.gdop,
 		session->gpsdata.satellites_used,buf2);
-        mask |= HDOP_SET | VDOP_SET | PDOP_SET | TDOP_SET | GDOP_SET | STATUS_SET | MODE_SET;
+        mask |= HDOP_SET | VDOP_SET | PDOP_SET | TDOP_SET | GDOP_SET | STATUS_SET | MODE_SET | USED_SET;
 	break;
     case 0x6e:		/* Synchronized Measurements */
 	break;
@@ -460,9 +470,9 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	session->gpsdata.newdata.altitude  = getd(buf,16);
 	d1 = getd(buf,24);			/* clock bias */
 	f1 = getf(buf,32);			/* time-of-fix */
-	if (session->tsip.gps_week) {
+	if (session->driver.tsip.gps_week) {
 	    session->gpsdata.newdata.time = session->gpsdata.sentence_time =
-		gpstime_to_unix((int)session->tsip.gps_week, f1) - session->context->leap_seconds;
+		gpstime_to_unix((int)session->driver.tsip.gps_week, f1) - session->context->leap_seconds;
 	    mask |= TIME_SET;
 	}
 	gpsd_report(4, "GPS DP LLA %f %f %f\n",session->gpsdata.newdata.latitude,session->gpsdata.newdata.longitude,session->gpsdata.newdata.altitude);
@@ -539,7 +549,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		session->context->leap_seconds = (int)u4;
 		session->context->valid |= LEAP_SECOND_VALID;
 	    }
-	    session->tsip.gps_week = s4;
+	    session->driver.tsip.gps_week = s4;
 	    session->gpsdata.newdata.time = session->gpsdata.sentence_time =
 		gpstime_to_unix((int)s4, ul1 * 1e-3) - session->context->leap_seconds;
 	    mask |= TIME_SET | LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET | STATUS_SET | MODE_SET | CYCLE_START_SET; 
@@ -558,7 +568,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    s3 = getsw(buf,23);			/* north velocity */
 	    s4 = getsw(buf,25);			/* up velocity */
 	    gpsd_report(4, "CSP %u %d %u %u %d %u %d %d %d %d\n",ul1,s1,u1,u2,sl1,ul2,sl3,s2,s3,s4);
-	    session->tsip.gps_week = s1;
+	    session->driver.tsip.gps_week = s1;
 	    if ((int)u1 > 10) {
 		session->context->leap_seconds = (int)u1;
 		session->context->valid |= LEAP_SECOND_VALID;
@@ -599,6 +609,20 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    gpsd_report(4,"Unhandled TSIP superpacket type 0x%02x\n",u1);
 	}
 	break;
+    case 0xbb:		/* Navigation Configuration */
+	if (len != 40)
+	    break;
+	u1 = getub(buf,0);			/* Subcode */
+	u2 = getub(buf,1);			/* Operating Dimension */
+	u3 = getub(buf,2);			/* DGPS Mode */
+	u4 = getub(buf,3);			/* Dynamics Code */
+	f1 = getf(buf,5);			/* Elevation Mask */
+	f2 = getf(buf,9);			/* AMU Mask */
+	f3 = getf(buf,13);			/* DOP Mask */
+	f4 = getf(buf,17);			/* DOP Switch */
+	u5 = getub(buf,21);			/* DGPS Age Limit */
+	gpsd_report(4, "Navigation Configuration %u %u %u %u %f %f %f %f %u\n",u1,u2,u3,u4,f1,f2,f3,f4,u5);
+	break;
     default:
 	gpsd_report(4,"Unhandled TSIP packet type 0x%02x\n",id);
 	break;
@@ -607,42 +631,64 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
     /* see if it is time to send some request packets for reports that */
     /* the receiver won't send at fixed intervals */
 
-    if ((now - session->tsip.last_41) > 5) {
+    if ((now - session->driver.tsip.last_41) > 5) {
 	/* Request Current Time */
 	(void)tsip_write(session->gpsdata.gps_fd, 0x21, buf, 0);
-	session->tsip.last_41 = now;
+	session->driver.tsip.last_41 = now;
     }
 
-    if ((now - session->tsip.last_6d) > 5) {
+    if ((now - session->driver.tsip.last_6d) > 5) {
 	/* Request GPS Receiver Position Fix Mode */
 	(void)tsip_write(session->gpsdata.gps_fd, 0x24, buf, 0);
-	session->tsip.last_6d = now;
+	session->driver.tsip.last_6d = now;
     }
 
-    if ((now - session->tsip.last_5c) >= 5) {
+    if ((now - session->driver.tsip.last_5c) >= 5) {
 	/* Request Current Satellite Tracking Status */
 	putbyte(buf,0,0x00);		/* All satellites */
 	(void)tsip_write(session->gpsdata.gps_fd, 0x3c, buf, 1);
-	session->tsip.last_5c = now;
+	session->driver.tsip.last_5c = now;
     }
 
     return mask;
 }
 
+static gps_mask_t tsip_parse_input(struct gps_device_t *session)
+{
+    gps_mask_t st;
+
+    if (session->packet_type == TSIP_PACKET){
+	st = tsip_analyze(session);
+	session->gpsdata.driver_mode = 1;
+	return st;
+#ifdef EVERMORE_ENABLE
+    } else if (session->packet_type == EVERMORE_PACKET) {
+	(void)gpsd_switch_driver(session, "EverMore binary");
+	st = evermore_parse(session, session->outbuffer, session->outbuflen);
+	session->gpsdata.driver_mode = 0;
+	return st;
+#endif /* EVERMORE_ENABLE */
+    } else
+	return 0;
+}
+
 /* this is everything we export */
 struct gps_type_t tsip_binary =
 {
-    "Trimble TSIP",	/* full name of type */
-    NULL,		/* no probe */
-    NULL,		/* only switched to by some other driver */
-    tsip_initializer,	/* initialization */
-    packet_get,		/* how to get a packet */
-    tsip_analyze,	/* read and parse message packets */
-    NULL,		/* send DGPS correction */
-    tsip_speed_switch,	/* change baud rate */
-    NULL,		/* no mode switcher */
-    NULL,		/* caller needs to supply a close hook */
-    1,			/* updates every second */
+    .typename       = "Trimble TSIP",	/* full name of type */
+    .trigger        = NULL,		/* no trigger */
+    .channels       = TSIP_CHANNELS,	/* consumer-grade GPS */
+    .probe          = NULL,		/* no probe */
+    .initializer    = tsip_initializer,	/* initialization */
+    .get_packet     = packet_get,	/* use the generic packet getter */
+    .parse_packet   = tsip_parse_input,	/* parse message packets */
+    .rtcm_writer    = NULL,		/* doesn't accept DGPS corrections */
+    .speed_switcher = tsip_speed_switch,/* change baud rate */
+    .mode_switcher  = NULL,		/* no mode switcher */
+    .rate_switcher  = NULL,		/* no rate switcher */
+    .cycle_chars    = -1,		/* not relevant, no rate switcher */
+    .wrapup         = tsip_wrapup,	/* restore comms parameters */
+    .cycle          = 1,		/* updates every second */
 };
 
 #endif /* TSIP_ENABLE */
